/*
  Copyright (C) 2009 SWOAG Technology <www.swoag.com>

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  LIMOX IPC packet parser routine for outputmix sub-system.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-09-17 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "limthread.h"
#include "limmemory.h"
#include "limqlist.h"
#include "OpenMAXAL.h"
#include "ox-session.h"
#include "common-be.h"
#include "output-mix-be.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static inline ox_omix_t * omix_init (ox_session_t * ox)
{
    ox_omix_t * omix = NULL;

    omix = lim_mem_pool_get_buf(ox->pool, sizeof(ox_omix_t));
    ox->omix = omix;
    omix->ox = ox;

    return omix;
}

static inline int omix_create (
    ox_session_t * ox,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    ox_omix_t * omix;
    int ret = -1;

    rsps->header.bytes = 0;

    omix = omix_init(ox);
    if (NULL == omix)
    {
        LIMERR("unable to initialize output mix.\n");
        goto bail;
    }

    ret = 0;
 bail:
    return ret;
}

static inline int omix_realize (
    ox_session_t * ox,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    ox_omix_t * omix = (ox_omix_t *)ox->omix;
    int ret = -1;

    rsps->header.bytes = 0;
    if (NULL == omix)
    {
        LIMERR("omix is not available.\n");
        goto bail;
    }
    ox_omix_realize(omix);
    ret = 0;
 bail:
    return ret;
}

static inline int omix_destroy (
    ox_session_t * ox,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    ox_omix_t * omix = (ox_omix_t *)ox->omix;

    rsps->header.bytes = 0;
    return ox_omix_destroy(omix);
}

int omix_parse_packet (
    ox_session_t * ox,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    switch (rqst->header.id.value)
    {
    case OX_RQST_OUTPUT_MIX_CREATE :
        return omix_create(ox, rqst, rsps);

    case OX_RQST_OUTPUT_MIX_REALIZE :
        return omix_realize(ox, rqst, rsps);

    case OX_RQST_OUTPUT_MIX_DESTROY :
        return omix_destroy(ox, rqst, rsps);

    default :
        LIMWARN("unkown outputmix message, segment: %d, id: %d\n",
               MSG_SEGMENT(rqst->header.id.value),
               MSG_ID(rqst->header.id.value));
        rsps->header.bytes = 0;
        break;
    }
    return 0;
}
