/*
  Copyright (C) 2009 - 2010 SWOAG Technology <www.swoag.com>

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  LIMOX IPC packet parser routine.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-09-11 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "ox-session.h"
#include "engine-be.h"
#include "player-be.h"
#include "output-mix-be.h"

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#define print_id_name(id)
#else
typedef struct
{
    uint32_t id;
    char * name;
} id_name_map_t;

static const id_name_map_t ID_NAME_MAP[] = {
    {OX_RQST_ENGINE_PING,                 "ENGINE PING"},
    {OX_RQST_OUTPUT_MIX_CREATE,           "OUTPUT MIX CREATE"},
    {OX_RQST_OUTPUT_MIX_REALIZE,          "OUTPUT MIX REALIZE"},
    {OX_RQST_OUTPUT_MIX_DESTROY,          "OUTPUT MIX DESTROY"},
    {OX_RQST_PLAYER_CREATE,               "PLAYER CREATE"},
    {OX_RQST_PLAYER_REALIZE,              "PLAYER REALIZE"},
    {OX_RQST_PLAYER_DESTROY,              "PLAYER DESTROY"},
    {OX_RQST_PLAYER_PLAY,                 "PLAYER PLAY"},
    {OX_RQST_PLAYER_PAUSE,                "PLAYER PAUSE"},
    {OX_RQST_PLAYER_STOP,                 "PLAYER STOP"},
    {OX_RQST_PLAYER_GET_POSITION,         "PLAYER GET POSITION"},
    {OX_RQST_PLAYER_SET_POSITION,         "PLAYER SET POSITION"},
    {OX_RQST_PLAYER_REGISTER_CALLBACK,    "PLAYER REGISTER CALLBACK"},
    {OX_RQST_PLAYER_SET_CALLBACK_EVENTS_MASK,
     "PLAYER SET CALLBACK EVENTS MASK"},
    {-1, NULL}
};

static void print_id_name(uint32_t id)
{
    id_name_map_t * pmap = (id_name_map_t *)&ID_NAME_MAP[0];

    while (pmap->name)
    {
        if (pmap->id == id)
        {
            LIMDBG("RQST: %X - %s\n", id, pmap->name);
            return;
        }
        pmap++;
    }
    LIMDBG("RQST: %X - unknown message id\n", id);
}
#endif //LIMDBG

int ox_parse_packet (
    ox_session_t * ox,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    print_id_name(rqst->header.id.value);

    switch (MSG_SEGMENT(rqst->header.id.value))
    {
    case SGMT_ENGINE :
        return engine_parse_packet(ox, rqst, rsps);

    case SGMT_PLAYER :
        return player_parse_packet(ox, rqst, rsps);

    case SGMT_OUTPUT_MIX :
        return omix_parse_packet(ox, rqst, rsps);

    default :
        LIMWARN("unkown IPC message, segment: %d, id: %d\n",
               MSG_SEGMENT(rqst->header.id.value),
               MSG_ID(rqst->header.id.value));
        rsps->header.bytes = 0;
        break;
    }
    return 0;
}
