/*
  Copyright (C) 2009 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX AL object deep copy implementation.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-09-19 MG
*/

#include "limlog.h"
#include "limipc.h"
#include "limdeepcopy.h"
#include "OpenMAXAL.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static inline int cp_locator(void * dst, const void * src, void * ptroff)
{
    XAuint32 * type = (XAuint32 *)src;
    int bytes = 0;

    switch (*type)
    {
    case XA_DATALOCATOR_URI :
        {
            XADataLocator_URI * uri_src = (XADataLocator_URI *)src;
            XADataLocator_URI * uri_dst = (XADataLocator_URI *)dst;
            int len;

            *uri_dst = *uri_src;
            bytes += sizeof(XADataLocator_URI);

            dst += bytes;
            uri_dst->pURI = dst;
            len = strlen((char *)uri_src->pURI + (ptroff - NULL)) + 1;
            LIMDBG("client uri = %s\n", (char *)uri_src->URI + (ptroff - NULL));
            if (len > 512)
            {
                LIMERR("URI too long : %d\n", len);
                bytes = 0;
            } else
            {
                memcpy(dst, uri_src->pURI + (ptroff - NULL), len);
                bytes += len;
            }
        }
        break;

    case XA_DATALOCATOR_OUTPUTMIX :
        {
            XADataLocator_OutputMix * omix_src = (XADataLocator_OutputMix *)src;
            XADataLocator_OutputMix * omix_dst = (XADataLocator_OutputMix *)dst;

            *omix_dst = *omix_src;
            bytes += sizeof(XADataLocator_OutputMix);
        }
        break;

    case XA_DATALOCATOR_ADDRESS :
    case XA_DATALOCATOR_IODEVICE :
    case XA_DATALOCATOR_NATIVEDISPLAY :
    case XA_DATALOCATOR_RESERVED6 :
    case XA_DATALOCATOR_RESERVED7 :
        LIMWARN("unsupported data locator type: %d\n", (int)*type);
        break;

    default :
        LIMERR("unrecognized data locator type: %d\n", (int)*type);
        break;
    }

    if (bytes > 0) bytes = PTR_PADDING(bytes);

    return bytes;
}

static inline int cp_format(void * dst, const void * src, void * ptroff)
{
    XAuint32 * type = (XAuint32 *)src;
    int bytes = 0;

    switch (*type)
    {
    case XA_DATAFORMAT_MIME :
        {
            XADataFormat_MIME * mime_src = (XADataFormat_MIME *)src;
            XADataFormat_MIME * mime_dst = (XADataFormat_MIME *)dst;
            int len;

            *mime_dst = *mime_src;
            bytes += sizeof(XADataFormat_MIME);

            dst += bytes;
            mime_dst->pMimeType = dst;
            len = strlen((char *)mime_src->pMimeType + (ptroff - NULL)) + 1;
            LIMDBG("client mimeType = %s\n",
                   (char *)mime_src->mimeType + (ptroff - NULL));
            if (len > 128)
            {
                LIMERR("MIME type too long : %d?\n", len);
                bytes = 0;
            } else
            {
                memcpy(dst, mime_src->pMimeType + (ptroff - NULL), len);
                bytes += len;
            }
        }
        break;

    case XA_DATAFORMAT_PCM :
    case XA_DATAFORMAT_RAWIMAGE :
        LIMWARN("unsupported data format type: %d\n", (int)*type);
        break;

    default :
        LIMERR("unrecognized data format: %d\n", (int)*type);
        break;
    }

    if (bytes > 0) bytes = PTR_PADDING(bytes);

    return bytes;
}

int deep_copy_src_snk (void * dst, const void * src, void * dptr, void * ptroff)
{
    int bytes, len;
    XADataSource * ds_src;
    XADataSource * ds_dst;

    if (!src) return 0;

    ds_src = (XADataSource *)src;
    ds_dst = (XADataSource *)dst;
    len = 0;

    if (ds_src->pLocator)
    {
        ds_dst->pLocator = dptr;
        bytes = cp_locator(dptr, ds_src->pLocator + (ptroff - NULL), ptroff);
        if (bytes <= 0)
        {
            ds_dst->pLocator = NULL;
            goto bail;
        }
        dptr += bytes;
        len = bytes;
    }

    if (ds_src->pFormat)
    {
        ds_dst->pFormat = dptr;
        bytes = cp_format(dptr, ds_src->pFormat + (ptroff - NULL), ptroff);
        if (bytes <= 0)
        {
            ds_dst->pFormat = NULL;
            goto bail;
        }
        len += bytes;
    }

 bail:
    return len;
}
