/*
  Copyright (C) 2009 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX AL ENGINE: callback support implementation.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-09-22 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "limthread.h"
#include "OpenMAXAL.h"
#include "engine-fe.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#define print_id_name(id)
#else
typedef struct
{
    uint32_t id;
    char * name;
} id_name_map_t;

static const id_name_map_t ID_NAME_MAP[] = {
    {OX_CALLBACK_PLAY,                    "PLAY CALLBACK"},
    {-1, NULL}
};

static void print_id_name(uint32_t id)
{
    id_name_map_t * pmap = (id_name_map_t *)&ID_NAME_MAP[0];

    while (pmap->name)
    {
        if (pmap->id == id)
        {
            LIMDBG("RQST: %X - %s\n", id, pmap->name);
            return;
        }
        pmap++;
    }
    LIMDBG("RQST: %X - unknown message id\n", id);
}
#endif //LIMDBG

void  engine_callback_parser(ipc_packet_t * packet)
{
    print_id_name(packet->header.id.value);

    switch (packet->header.id.value)
    {
    case OX_CALLBACK_PLAY :
        {
            ox_callback_play_t * cb;
            cb = (ox_callback_play_t *)packet->dptr;
            cb->func(cb->caller, cb->context, cb->event);
        }
        break;

    default :
        LIMWARN("unkown IPC callback message, segment: %d, id: %d\n",
               MSG_SEGMENT(packet->header.id.value),
               MSG_ID(packet->header.id.value));
        break;
    }
}
