/*
  Copyright (C) 2010 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX AL ENGINE: ipc stub for thread communication.

  REVISION:

    1) initial creation. ----------------------------------------- 2010-04-06 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "limthread.h"
#include "OpenMAXAL.h"
#include "engine-fe.h"
#include "ox-session.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

int engine_ipc_init (engine_t * engine)
{
    engine_ipc_t * ipc = &engine->ipc;

    lthread_mutex_init(&ipc->mutex);
    lthread_cond_init(&ipc->cond);

    ipc->sid = ipc_session_create(-1);
    if (NULL == ipc->sid)
    {
        LIMERR("unable to initialize session.\n");
        return -1;
    } else
    {
        if (ox_session_init(ipc))
        {
            LIMERR("unable to create limox session.\n");
            ipc_session_destroy(ipc->sid);
            lthread_mutex_destroy(&ipc->mutex);
            lthread_cond_destroy(&ipc->cond);
            return -1;
        }
    }

    return 0;
}

int engine_ipc_destroy (engine_t * engine)
{
    engine_ipc_t * ipc = &engine->ipc;

    ipc_session_destroy(ipc->sid);
    ox_session_destroy(NULL);

    lthread_mutex_destroy(&ipc->mutex);
    lthread_cond_destroy(&ipc->cond);

    return 0;
}

int engine_ipc_send_packet (
    engine_t * engine,
    ipc_packet_t * rqst,
    ipc_packet_t * rsps)
{
    int ret = 0;
    engine_ipc_t * ipc = &engine->ipc;

    lthread_mutex_lock(&ipc->mutex);

    if (0 > ox_session_parse_packet(NULL, rqst, rsps))
    {
        LIMERR("unable to send packet.\n");
        ret = -1;
    }

    lthread_mutex_unlock(&ipc->mutex);
    return ret;
}

int engine_ipc_ping (engine_t * engine)
{
    ipc_packet_t pkt;

    pkt.header.id.value = OX_RQST_ENGINE_PING;
    pkt.header.bytes = 0;
    return engine_ipc_send_packet(engine, &pkt, NULL);
}
