/*
  Copyright (C) 2009 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX AL ENGINE: engine interface implementation.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-09-03 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "limthread.h"
#include "OpenMAXAL.h"
#include "engine-fe.h"
#include "output-mix-fe.h"
#include "player-fe.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static const XAInterfaceID * supported_engine_interfaces[] = {
    &XA_IID_OBJECT,
    &XA_IID_ENGINE,
};


XAuint32 engine_query_num_supported_interfaces (void)
{
    return sizeof(supported_engine_interfaces) / sizeof(XAInterfaceID *);
}

XAresult engine_query_interface_id (
    XAuint32 index,
    XAInterfaceID * pInterfaceId)
{
    if (index >= sizeof(supported_engine_interfaces) / sizeof(XAInterfaceID *))
    {
        LIMWARN("%s\n", xa_error_string(XA_RESULT_PARAMETER_INVALID));
        return XA_RESULT_PARAMETER_INVALID;
    }

    *pInterfaceId = *(supported_engine_interfaces[index]);

    return XA_RESULT_SUCCESS;
}

void engine_interface_init (engine_t * engine)
{
    struct XAEngineItf_ * itf;

    engine->engineItf.engine = engine;

    itf =
    engine->engine_itf = &engine->engineItf.itf;

    itf->CreateMediaPlayer = player_obj_create;
    itf->CreateOutputMix = output_mix_obj_create;
}
