/*
  Copyright (C) 2010 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX AL PLAYER: seek interface implementation.

  REVISION:

    1) initial creation. ----------------------------------------- 2010-04-08 MG
*/

#include "limoa-log.h"
#include "limipc.h"
#include "limthread.h"
#include "OpenMAXAL.h"
#include "engine-fe.h"
#include "player-fe.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static XAresult SetPosition (
    XASeekItf self,
    XAmillisecond pos,
    XAuint32 seekMode)
{
    XAresult result = XA_RESULT_PARAMETER_INVALID;
    player_t * player = ((seek_itf_t *)(*self))->player;
    engine_t * engine = player->engine;
    ipc_packet_t rqst;
    ipc_packet_t rsps;
    ox_rqst_player_set_position_t player_rqst;
    ipc_standard_rsps_t * player_rsps;

    ipc_init_request(&rqst, OX_RQST_PLAYER_SET_POSITION,
                     sizeof(ox_rqst_player_set_position_t),
                     &player_rqst);
    player_rqst.pos = pos;
    player_rqst.seek_mode = seekMode;
    if (engine_ipc_send_packet(engine, &rqst, &rsps))
    {
        LIMERR("unable to send player set position request.\n");
        goto bail;
    }
    player_rsps = rsps.dptr;
    result = player_rsps->result;
    ipc_release_packet(engine->ipc.sid, &rsps);

 bail:
    return result;
}

static XAresult SetLoop (
    XASeekItf self,
    XAboolean loopEnable,
    XAmillisecond startPos,
    XAmillisecond endPos)
{
    LIMWARN("API not implemented.\n");
    return XA_RESULT_SUCCESS;
}

static XAresult GetLoop (
    XASeekItf self,
    XAboolean * pLoopEnabled,
    XAmillisecond * pStartPos,
    XAmillisecond * pEndPos)
{
    LIMWARN("API not implemented.\n");
    return XA_RESULT_SUCCESS;
}

void seek_interface_init (player_t * player)
{
    struct XASeekItf_ * itf;

    player->seekItf.player = player;

    itf =
    player->seek_itf = &player->seekItf.itf;

    itf->SetPosition = SetPosition;
    itf->SetLoop = SetLoop;
    itf->GetLoop = GetLoop;
}
