/*
  Copyright (C) 2009 - 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL debug helper code.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-11-20 MG
*/

#include "limoi-log.h"
#include "limoi.h"
#include "component.h"

const char * limoi_omx_error_string (OMX_ERRORTYPE res)
{
    switch (res)
    {
    case OMX_ErrorNone :
        return "None";
    case OMX_ErrorInsufficientResources :
        return "InsufficientResources";
    case OMX_ErrorUndefined :
        return "Undefined";
    case OMX_ErrorInvalidComponentName :
        return "InvalidComponentName";
    case OMX_ErrorComponentNotFound :
        return "ComponentNotFound";
    case OMX_ErrorInvalidComponent :
        return "InvalidComponent";
    case OMX_ErrorBadParameter :
        return "BadParameter";
    case OMX_ErrorNotImplemented :
        return "NotImplemented";
    case OMX_ErrorUnderflow :
        return "Underflow";
    case OMX_ErrorOverflow :
        return "Overflow";
    case OMX_ErrorHardware :
        return "Hardware";
    case OMX_ErrorInvalidState :
        return "InvalidState";
    case OMX_ErrorStreamCorrupt :
        return "StreamCorrupt";
    case OMX_ErrorPortsNotCompatible :
        return "PortsNotCompatible";
    case OMX_ErrorResourcesLost :
        return "ResourcesLost";
    case OMX_ErrorNoMore :
        return "NoMore";
    case OMX_ErrorVersionMismatch :
        return "VersionMismatch";
    case OMX_ErrorNotReady :
        return "NotReady";
    case OMX_ErrorTimeout :
        return "TimeOut";
    case OMX_ErrorSameState :
        return "SameState";
    case OMX_ErrorResourcesPreempted :
        return "ResourcePreempted";
    case OMX_ErrorPortUnresponsiveDuringAllocation :
        return "PortUnresponsiveDuringAllocation";
    case OMX_ErrorPortUnresponsiveDuringDeallocation :
        return "PortUnresponsiveDuringDeallocation";
    case OMX_ErrorPortUnresponsiveDuringStop :
        return "PortUnresponsiveDuringStop";
    case OMX_ErrorIncorrectStateTransition :
        return "IncorrectStateTransition";
    case OMX_ErrorIncorrectStateOperation :
        return "IncorrectStateOperation";
    case OMX_ErrorUnsupportedSetting :
        return "UnsupportedSetting";
    case OMX_ErrorUnsupportedIndex :
        return "UnsupportedIndex";
    case OMX_ErrorBadPortIndex :
        return "BadPortIndex";
    case OMX_ErrorPortUnpopulated :
        return "PortUnpopulated";
    case OMX_ErrorComponentSuspended :
        return "ComponentSuspended";
    case OMX_ErrorDynamicResourcesUnavailable :
        return "DynamicResourcesUnavailable";
    case OMX_ErrorMbErrorsInFrame :
        return "MbErrorsInFrame";
    case OMX_ErrorFormatNotDetected :
        return "FormatNotDetected";
    case OMX_ErrorContentPipeOpenFailed :
        return "ContentPipeOpenFailed";
    case OMX_ErrorContentPipeCreationFailed :
        return "ContentPipeCreationFailed";
    case OMX_ErrorSeperateTablesUsed :
        return "SeperateTablesUsed";
    case OMX_ErrorTunnelingUnsupported :
        return "TunnelingUnsupported";
    default :
        break;
    }
    return "Unkown";
}

const char * limoi_omx_state_string (OMX_STATETYPE state)
{
    switch (state)
    {
    case OMX_StateInvalid :
        return "Invalid";
    case OMX_StateLoaded :
        return "Loaded";
    case OMX_StateIdle :
        return "Idle";
    case OMX_StateExecuting :
        return "Executing";
    case OMX_StatePause :
        return "Pause";
    case OMX_StateWaitForResources :
        return "WaitForResources";
    case OMX_StateMax :
        return "StateMax";
    default :
        break;
    }

    if (state >= OMX_StateVendorStartUnused)
        return "Vendor";

    if (state >= OMX_StateKhronosExtensions)
        return "KhronosExtensions";

    return "Unknown";
}

const char * limoi_omx_event_string (OMX_EVENTTYPE event)
{
    switch (event)
    {
    case OMX_EventCmdComplete :
        return "CmdComplete";
    case OMX_EventError :
        return "Error";
    case OMX_EventPortSettingsChanged :
        return "PortSettingsChanged";
    case OMX_EventBufferFlag :
        return "BufferFlag";
    case OMX_EventResourcesAcquired :
        return "ResourcesAcquired";
    case OMX_EventComponentResumed :
        return "ComponentResumed";
    case OMX_EventDynamicResourcesAvailable :
        return "DynamicResourcesAvailable";
    case OMX_EventPortFormatDetected :
        return "PortFormatDetected";
    default :
        break;
    }
    return "Unknown";
}

const char * limoi_omx_command_string (OMX_COMMANDTYPE cmd)
{
    switch (cmd)
    {
    case OMX_CommandStateSet :
        return "OMX_CommandStateSet";
    case OMX_CommandPortDisable :
        return "OMX_CommandPortDisable";
    case OMX_CommandPortEnable :
        return "OMX_CommandPortEnable";
    case OMX_CommandFlush :
        return "OMX_CommandFlush";
    case OMX_CommandMarkBuffer :
        return "OMX_CommandMarkBuffer";
    default :
        break;
    }
    return "Unknown command";
}

void limoi_omx_port_status (OMX_HANDLETYPE hComp, OMX_U32 index)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    limoi_port_t * limoi_port;
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_PARAM_BUFFERSUPPLIERTYPE * supplier;

    limoi_port = limoi_component_get_port(hComp, index);
    if (NULL == limoi_port)
    {
        LIMERR("% unable to access port %u\n", base->name, index);
        return;
    }

    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);

    printf("%s port %u status:\n"
           "\ttunneled         : %s\n"
           "\ttunnel supplier  : %s\n"
           "\tdirection        : %s\n"
           "\tshare policy     : %s\n"
           "\tnumber of shares : %d (%d %d %d %d)\n",
           base->name, index,
           limoi_port->tunnel_comp? "YES" : "NO",
           ((supplier->eBufferSupplier == OMX_BufferSupplyInput
             && port->eDir == OMX_DirInput)
            || (supplier->eBufferSupplier == OMX_BufferSupplyOutput
                && port->eDir == OMX_DirOutput))? "YES" : "NO",
           (port->eDir == OMX_DirInput)? "IN" : "OUT",
           limoi_port->buffer_share.type == LIMOX_BufferShareDisabled?
           "DISABLED" :
           limoi_port->buffer_share.type == LIMOX_BufferShareSupplier?
           "SUPPLIER" : "CONSUMER",
           limoi_port->buffer_share.share_num,
           limoi_port->buffer_share.ports[0], limoi_port->buffer_share.ports[1],
           limoi_port->buffer_share.ports[2], limoi_port->buffer_share.ports[3]);
}
