/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL component client event handling subroutines.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-07-22 MG
*/
#define _GNU_SOURCE
#include "limoi.h"
#include "component.h"
#include "string.h"
#include <stdarg.h>

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

void event_deliver (
    OMX_HANDLETYPE hComp,
    OMX_EVENTTYPE eEvent,
    OMX_U32 nData1,
    OMX_U32 nData2,
    OMX_PTR pEventData)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    limoi_component_client_event_t * event;

    if (!base->callbacks.EventHandler)
    {
        LIMERR("%s mandatory component event callback handle is NULL.\n",
               base->name);
    } else
    {
        LIMDBG("%s enqueuing event back to client:\n"
               "\teEvent      : %s\n"
               "\tnData1      : %u\n"
               "\tnData2      : %u\n"
               "\tpAppPrivate : %p\n"
               "\tpEventData  : %p\n",
               base->name,
               limoi_omx_event_string(eEvent), nData1, nData2,
               ((OMX_COMPONENTTYPE *)hComp)->pApplicationPrivate,
               pEventData);

        lthread_mutex_lock(&base->event_mutex);
        event = limset_new(base->event_set);
        if (NULL == event)
        {
            LIMERR("event set full, client stopped responding?\n");
            lthread_mutex_unlock(&base->event_mutex);
            return;
        }
        event->threadId = lthread_self();
        event->eEvent = eEvent;
        event->nData1 = nData1;
        event->nData2 = nData2;
        event->pEventData = pEventData;
        limset_commit(base->event_set, event);
        lthread_mutex_unlock(&base->event_mutex);
    }
}

void event_deliver_real (OMX_HANDLETYPE hComp)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    limoi_component_client_event_t event;
    limoi_component_client_event_t * eventp;
    limoi_component_client_event_t * eventp_save;

    lthread_mutex_lock(&base->event_mutex);

    eventp = limset_next(base->event_set, NULL);
    while (eventp)
    {
        event = *eventp;
        eventp_save = eventp;
        eventp = limset_next(base->event_set, eventp_save);

        if (event.threadId == lthread_self())
        {
            limset_consume(base->event_set, eventp_save);
            lthread_mutex_unlock(&base->event_mutex);

            LIMDBG("%s delivering event back to client:\n"
                   "\teEvent      : %s\n"
                   "\tnData1      : %u\n"
                   "\tnData2      : %u\n"
                   "\tpAppPrivate : %p\n"
                   "\tpEventData  : %p\n",
                   base->name,
                   limoi_omx_event_string(event.eEvent),
                   event.nData1, event.nData2,
                   ((OMX_COMPONENTTYPE *)hComp)->pApplicationPrivate,
                   event.pEventData);

            base->callbacks.EventHandler
                (hComp, ((OMX_COMPONENTTYPE *)hComp)->pApplicationPrivate,
                 event.eEvent, event.nData1, event.nData2, event.pEventData);

            lthread_mutex_lock(&base->event_mutex);
        }
    }

    lthread_mutex_unlock(&base->event_mutex);
}

void limoi_component_deliver_event (
    OMX_HANDLETYPE hComp,
    OMX_EVENTTYPE eEvent,
    OMX_U32 nData1,
    OMX_U32 nData2,
    OMX_PTR pEventData)
{
    return event_deliver(hComp, eEvent, nData1, nData2, pEventData);
}
