/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL component port support subroutines.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-07-22 MG
*/
#define _GNU_SOURCE
#include "limoi.h"
#include "component.h"
#include "string.h"
#include <stdarg.h>

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

#if 0
static void print_port_buffers (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    int ii = 0;
    list_t * list = limoi_port->buffers_list_head.next;

    while (list != &limoi_port->buffers_list_head)
    {
        LIMINFO("%s port %u buffer header %d [%p | %p].\n",
                base->name, limoi_port->index, ii++,
                ((limoi_port_buffer_t *)list)->header,
                ((limoi_port_buffer_t *)list)->header->pBuffer);
        list = list->next;
    }
}
#endif

//return: 0: nothing has been released
//        1: at least one buffer is released.
static int release_buffers (
    list_t * buffers_list_head)
{
    list_t * list = buffers_list_head->next;
    limoi_port_buffer_t * reg;

    if (list == buffers_list_head) return 0;

    while (list != buffers_list_head)
    {
        reg = (limoi_port_buffer_t *)list;

#if 0
        if (reg->allocated)
        {
            LIMDBG("release port buffer: %p\n", reg->header->pBuffer);
            //buffers were allocated from memory pool, which has
            //been either destroyed or reset prior to coming here,
            //thus no need for the 'free' action on individual buffer.
        }
#endif
        LIMDBG("release port buffer header: %p\n", reg->header);
        free((void *)reg->header);

        list = list->next;

        list_del(&reg->list);
        free(reg);
    }

    return 1;
}

void port_release (list_t * ports_list_head, OMX_HANDLETYPE hComp)
{
    list_t * list = ports_list_head->next;
    limoi_port_t * reg;

    while (list != ports_list_head)
    {
        reg = (limoi_port_t *)list;
        LIMDBG("releasing port %u\n", reg->index);

        //TODO: before releaseing buffers, waiting till buffers_q is empty.
        limq_destroy(reg->bufq);
        lim_mem_pool_destroy(reg->buf_mem_pool);
        lthread_mutex_destroy(&reg->mutex);
        lthread_cond_destroy(&reg->cond);

        LIMDBG("releasing buffers of port %u\n", reg->index);
        if (release_buffers(&reg->buffers_list_head))
        {
            LIMWARN("%s OMX_FreeBuffer not called?\n",
                    LIMOI_COMPONENT_PRIV(hComp)->name);
        }
        LIMDBG("releasing params of port %u\n", reg->index);
        param_release(&reg->params_list_head);

        list = list->next;
        list_del(&reg->list);
        free(reg);
    }
}

static int port_buffer_num (
    limoi_port_t * limoi_port)
{
    list_t * list = limoi_port->buffers_list_head.next;
    int num = 0;
    while (list != &limoi_port->buffers_list_head)
    {
        num++;
        if (num > LIMOI_COMPONENT_CMD_Q_DEPTH_MAX)
        {
            LIMERR("port buffer list corrupted?\n");
            break;
        }
        list = list->next;
    }
    return num;
}

static OMX_ERRORTYPE use_buffer (
    OMX_HANDLETYPE hComponent,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE** ppBufferHdr,
    OMX_PTR pAppPrivate,
    OMX_U32 nSizeBytes,
    OMX_U8* pBuffer)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_port_buffer_t * port_buf;
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_U32 nPortIndex = limoi_port->index;

    LIMDBG("%s port %u uses buffer %p\n",
           base->name, limoi_port->index, pBuffer);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);
    if (port->bPopulated == OMX_TRUE)
    {
        LIMWARN("%s port %u already populated.\n",
                base->name, limoi_port->index);
        ret = OMX_ErrorInvalidState;
        goto bail;
    }

    port_buf = malloc(sizeof(limoi_port_buffer_t));
    if (NULL == port_buf)
    {
        LIMERR("%s port %u out of memory.\n",
               base->name, limoi_port->index);
        ret = OMX_ErrorInsufficientResources;
        goto bail;
    }

    port_buf->header = malloc(sizeof(OMX_BUFFERHEADERTYPE));
    if (NULL == port_buf->header)
    {
        LIMERR("%s port %u out of memory.\n",
               base->name, limoi_port->index);
        ret = OMX_ErrorInsufficientResources;
        free(port_buf);
        goto bail;
    }

    if (NULL == pBuffer)
    {
        if (NULL == limoi_port->buf_mem_pool)
        {
            LIMDBG("%s port %d lazy buffer allocation, bytes: %d, count: %d\n",
                   base->name, limoi_port->index,
                   nSizeBytes, port->nBufferCountActual);
            //lazy port buffer allocation for non-tunneled port
            //buffer pool is allocated only when buffer is actually requested.
            limoi_port->buf_mem_pool = lim_mem_pool_create
                (nSizeBytes * port->nBufferCountActual);
            if (NULL == limoi_port->buf_mem_pool)
            {
                LIMERR("%s unable to allocate port %d memory.\n",
                       base->name, limoi_port->index);
            }
        }
        if (limoi_port->buf_mem_pool)
            pBuffer = lim_mem_pool_get_buf(limoi_port->buf_mem_pool, nSizeBytes);

        if (NULL == pBuffer)
        {
            LIMERR("%s port %u out of memory when allocating %u bytes.\n",
                   base->name, limoi_port->index, nSizeBytes);
            free(port_buf);
            free(port_buf->header);
            ret = OMX_ErrorInsufficientResources;
            goto bail;
        }
        port_buf->allocated = 1;
        LIMDBG("%s port %u buffer allocated as: %p\n", base->name,
               limoi_port->index, pBuffer);
    } else port_buf->allocated = 0;

    port_buf->header->pBuffer = pBuffer;
    limoi_set_spec_version(port_buf->header, sizeof(OMX_BUFFERHEADERTYPE));
    port_buf->header->nAllocLen = nSizeBytes;
    port_buf->header->nFilledLen = 0;
    port_buf->header->nOffset = 0;
    port_buf->header->nFlags = 0;
    port_buf->header->pAppPrivate = pAppPrivate;

	if (OMX_DirOutput == port->eDir)
	{
        port_buf->header->nOutputPortIndex = nPortIndex;
        port_buf->header->pOutputPortPrivate = pAppPrivate;
        port_buf->header->nInputPortIndex = 0x7fffffff;
    } else
    {
        port_buf->header->nInputPortIndex = nPortIndex;
        port_buf->header->pInputPortPrivate = pAppPrivate;
        port_buf->header->nOutputPortIndex = 0x7fffffff;
    }

    list_init(&port_buf->list);
    list_add(&port_buf->list, &limoi_port->buffers_list_head);

    *ppBufferHdr = port_buf->header;

    if (port_buffer_num(limoi_port) >= port->nBufferCountActual)
    {
        LIMDBG("%s port %u buffer allocated actual %u.\n",
               base->name, nPortIndex, port->nBufferCountActual);
        port->bPopulated = OMX_TRUE;
    }
    lthread_cond_broadcast(&limoi_port->cond);

    ASSERT(list_entries(&limoi_port->buffers_list_head) <=
           port->nBufferCountActual);

 bail:
    lthread_mutex_unlock(&limoi_port->mutex);

    return ret;
}

//caller must hold port mutex
static OMX_ERRORTYPE enqueue_this_buffer (
    OMX_HANDLETYPE hComponent,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE * pBuffer)
{
    OMX_BUFFERHEADERTYPE ** buf;

    buf = limq_enqueue_get(limoi_port->bufq);
    if (NULL == buf)
    {
        LIMERR("%s port %u buffer queue full.\n",
               LIMOI_COMPONENT_PRIV(hComponent)->name, limoi_port->index);
        return OMX_ErrorInsufficientResources;
    }

    *buf = pBuffer;
    limq_enqueue_put(limoi_port->bufq, buf);

    return OMX_ErrorNone;
}

int port_allocate_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    int ret = 0;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    int is_allocator = tunnel_is_allocator(hComp, limoi_port);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);

    LIMDBG("%s allocating port %u buffer.\n"
           "\tbEnabled   : %d\n"
           "\tbPopulated : %d\n",
           base->name, limoi_port->index,
           port->bEnabled, port->bPopulated);
    //if port is disabled, do not allocate buffer.
    //however, it is entirely possible that a disabled port gets its buffer
    //allocated, by its supplier port which is enabled.
    if (port->bEnabled == OMX_FALSE || port->bPopulated == OMX_TRUE) goto bail;

    if (is_allocator)
    {
        //allocate buffers
        //OMX_PARAM_PORTDEFINITIONTYPE tunneled_port;
        OMX_U32 count, bytes;

        LIMDBG("%s port %u allocates buffer as supplier.\n",
               base->name, port->nPortIndex);

        //supplier assign buffer size.
        //non-supplier don't care because buffer size info is carried
        //over through buffer header.
        lthread_mutex_unlock(&limoi_port->mutex);
        if (buffer_share_get_buffer_count_size
            (hComp, limoi_port, &count, &bytes))
        {
            LIMERR("%s unable to get tunneled port %d params.\n",
                   base->name, limoi_port->tunnel_port);
            lthread_mutex_lock(&limoi_port->mutex);
            ret = -1;
            goto bail;
        } else if (buffer_share_set_buffer_count_size
                   (hComp, limoi_port, count, bytes))
        {
            LIMERR("%s unable to set tunneled port %d params.\n",
                   base->name, limoi_port->tunnel_port);
            lthread_mutex_lock(&limoi_port->mutex);
            ret = -1;
            goto bail;
        }

        lthread_mutex_lock(&limoi_port->mutex);

        ASSERT(port->nBufferSize > 0);

        //supplier port initializes its buffer memory pool.
        limoi_port->buf_mem_pool = lim_mem_pool_create
            (port->nBufferSize * port->nBufferCountActual);
        if (NULL == limoi_port->buf_mem_pool)
        {
            LIMERR("%s unable to allocate port %d memory.\n",
                   base->name, limoi_port->tunnel_port);
            lthread_mutex_lock(&limoi_port->mutex);
            ret = -1;
            goto bail;
        }
    }

    if (tunnel_is_supplier(limoi_port))
    {
        int ii = 0;
        int retry_count = 0;

        while (ii++ < port->nBufferCountActual)
        {
            OMX_BUFFERHEADERTYPE * buf1;
            OMX_BUFFERHEADERTYPE * buf2;
            OMX_ERRORTYPE res;
            OMX_U8 * pBuffer;

            LIMDBG("%s allocating %u bytes buffer for port %u\n",
                   base->name, port->nBufferSize, limoi_port->index);

            if (!is_allocator)
            {
                //must be buffer sharing case
                if (limoi_port->buffer_share.type != LIMOX_BufferShareConsumer)
                {
                    LIMERR("%s unexpected buffer sharing error.\n", base->name);
                    ret = -1;
                    goto bail;
                }

                pBuffer = buffer_share_get_supplier_buffer
                    (hComp, limoi_port, ii - 1);
                if (pBuffer == NULL)
                {
                    //client must have enabled the port in wrong order
                    //supplier port must be enabled the first.
                    LIMERR("%s port %u unable to get supplier port %u buffer.\n",
                           base->name, limoi_port->index,
                           limoi_port->buffer_share.ports[0]);
                    ret = -1;
                    goto bail;
                }
            } else pBuffer = NULL;

            if (OMX_ErrorNone !=
                use_buffer(hComp, limoi_port, &buf1,
                           NULL, port->nBufferSize, pBuffer))
            {
                LIMERR("%s port %u out of memory.\n",
                       base->name, limoi_port->index);
                ret = -1;
                goto bail;
            }

        retry:
            LIMDBG("%s notify tunneled component to use buffer\n",
                   base->name);
            lthread_mutex_unlock(&limoi_port->mutex);
            res = OMX_UseBuffer(limoi_port->tunnel_comp, &buf2,
                                limoi_port->tunnel_port, NULL,
                                port->nBufferSize,
                                buf1->pBuffer);
            lthread_mutex_lock(&limoi_port->mutex);
            if (OMX_ErrorInvalidState == res)
            {
                LIMWARN("%s port %u tunnel component invalid state?\n",
                        base->name, limoi_port->index);
                //tunnel component reports invalid state, wait a bit
                //before reporting as error.
                if (retry_count++ < 60)
                {
                    usleep(50000);
                    goto retry;
                } else
                {
                    LIMERR("%s port %u tunnel component invalid state.\n",
                           base->name, limoi_port->index);
                    ret = -1;
                    goto bail;
                }
            } else if (OMX_ErrorNone != res)
            {
                LIMERR("%s tunnel component port %u out of memory.\n",
                       base->name,
                       limoi_port->tunnel_port);
                ret = -1;
                goto bail;
            }
        }
        goto bail;
    }

    //wait_for_buffers: to be populated if required ->nBufferActual nonzero
    while (port->nBufferCountActual)
    {
        LIMDBG("%s port %u buffer actual %u, min %u, populated %s\n",
               base->name, limoi_port->index,
               port->nBufferCountActual, port->nBufferCountMin,
               (port->bPopulated == OMX_TRUE)? "true" : "false");
        if (port->bPopulated == OMX_TRUE) break;
        if (port->bEnabled == OMX_FALSE)
        {
            LIMWARN("%s port %u is disabled while being populated.\n",
                    base->name, limoi_port->index);
            break;
        }

        if (lthread_cond_timedwait(&limoi_port->cond,
                                   &limoi_port->mutex, 5000))
        {
            LIMWARN("%s port %u allocate buffer timed out,\n"
                    "client or supplier busy?\n"
                    "\tactual : %u\n"
                    "\tminimum : %u\n"
                    "\tcurrent : %d\n",
                    base->name, limoi_port->index,
                    port->nBufferCountActual,
                    port->nBufferCountMin,
                    port_buffer_num(limoi_port));
            break;
        }
    }

 bail:
    lthread_cond_broadcast(&limoi_port->cond);
    lthread_mutex_unlock(&limoi_port->mutex);
    LIMDBG("%s allocating port %u buffer done.\n",
           base->name, limoi_port->index);
    return ret;
}

//tunneled output port: put buffers to corresponding queue.
static int activate_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    int ret = 0;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_BUFFERHEADERTYPE * buf;
    list_t * list;

    lthread_mutex_lock(&limoi_port->mutex);
    list = limoi_port->buffers_list_head.next;
    port = LIMOI_PORTDEF(limoi_port);

    LIMDBG("%s activating port %u buffer.\n", base->name, limoi_port->index);

    //bail if port gets disabled/depopulated before we get here.
    if (port->bEnabled == OMX_FALSE || port->bPopulated == OMX_FALSE)
    {
        LIMDBG("%s port %u not populated (%d) yet or disabled (%d).\n",
               base->name, limoi_port->index,
               port->bPopulated, port->bEnabled);
        goto bail;
    }

    if (limoi_port->tunnel_comp)
    {
        if (limq_entries(limoi_port->bufq))
        {
            LIMDBG("%s port %u populated (%d) and enabled (%d).\n",
                   base->name, limoi_port->index,
                   port->bPopulated, port->bEnabled);
        } else
        {
            while (list != &limoi_port->buffers_list_head)
            {
                buf = ((limoi_port_buffer_t *)list)->header;
                if (port->eDir == OMX_DirOutput)
                {
                    LIMDBG("%s fill this buffer of port %u.\n",
                           base->name, limoi_port->index);
                    buf->nInputPortIndex = limoi_port->tunnel_port;
                    if (OMX_ErrorNone !=
                        enqueue_this_buffer(hComp, limoi_port, buf))
                    {
                        LIMERR("%s unable to enqueue buffer.\n",
                               base->name);
                        ret = -1;
                        goto bail;
                    }
                } else
                {
                    buf->nOutputPortIndex = limoi_port->tunnel_port;
                }
                list = list->next;
            }
        }
    }
    limoi_port->activated = 1;
    lthread_cond_broadcast(&limoi_port->cond);

 bail:
    LIMDBG("%s activating port %u buffer done.\n"
           "\tbufq_entries : %d total: %u\n",
           base->name, limoi_port->index,
           limq_entries(limoi_port->bufq), port->nBufferCountActual);
    ASSERT(limq_entries(limoi_port->bufq) <= port->nBufferCountActual);

    lthread_mutex_unlock(&limoi_port->mutex);
    return ret;
}

OMX_ERRORTYPE port_activate_buffer_all (OMX_HANDLETYPE hComp)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    list_t * list;

    // loop through ports to activate tunnel buffers
    LIMDBG("%s activating tunnel buffers\n", base->name);

    list = base->ports_list_head.next;
    while (list != &base->ports_list_head)
    {
        if (activate_port_buffer(hComp, (limoi_port_t *)list))
        {
            LIMERR("%s unable to activate buffer for port: %u.\n",
                   base->name, ((limoi_port_t *)list)->index);
            return OMX_ErrorUndefined;
        }
        list = list->next;
    }

    return OMX_ErrorNone;
}

//depopulating port buffer.
//before this gets called, all buffers must have been deactivated, in other
//words, all buffers are out of the buffer queue and no longer referenced.
static int free_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_PARAM_BUFFERSUPPLIERTYPE * supplier;

    LIMDBG("%s free buffers of port %u\n", base->name, limoi_port->index);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);

    if (limoi_port->tunnel_comp)
    {
        if (buffer_share_is_supplier(hComp, limoi_port))
        {
            int ii, port_index;
            OMX_PARAM_PORTDEFINITIONTYPE * port2;
            limoi_port_t * share_port;

            for (ii = 0; ii < base->buffer_share_policy->num_share_pairs; ii++)
            {
                if (base->buffer_share_policy->ports[ii * 2]
                    != limoi_port->index) continue;

                port_index = base->buffer_share_policy->ports[ii * 2 + 1];

                lthread_mutex_unlock(&limoi_port->mutex);
                share_port = limoi_component_get_port(hComp, port_index);
                lthread_mutex_lock(&limoi_port->mutex);

                if (NULL == share_port)
                {
                    LIMWARN("%s unable to get share port %u.\n",
                            base->name, port_index);
                    continue;
                }

                port2 = LIMOI_PORTDEF(share_port);
                if (port2->bPopulated)
                {
                    //all ports that share buffers from this port must
                    //have been freed first. Port operation are serialized
                    //by component, if we ever reach here, it is a BUG, or
                    //a client calling sequence error.
                    //ports that share buffers form a strong dependency.
                    LIMFATAL("%s port %u sharing port %u not freed yet,\n"
                             "check your calling sequence.\n", base->name,
                             limoi_port->index, port_index);
                    break;
                }
            }
        }

        if ((supplier->eBufferSupplier == OMX_BufferSupplyInput
             && port->eDir == OMX_DirInput)
            || (supplier->eBufferSupplier == OMX_BufferSupplyOutput
                && port->eDir == OMX_DirOutput))
        {
            list_t * list = limoi_port->buffers_list_head.next;

            //release all buffers for tunneled port by the buffer supplier.
            while (list != &limoi_port->buffers_list_head)
            {
                LIMDBG("%s port %u freeing buffer: %p.\n",
                       base->name, limoi_port->index,
                       ((limoi_port_buffer_t *)list)->header->pBuffer );
                lthread_mutex_unlock(&limoi_port->mutex);
                OMX_FreeBuffer(limoi_port->tunnel_comp,
                               limoi_port->tunnel_port,
                               ((limoi_port_buffer_t *)list)->header);
                lthread_mutex_lock(&limoi_port->mutex);
                list = list->next;
            }

            //now release buffers of port itself
            //tunnel supplier can share buffers from other port,
            //where buf_mem_pool would remain NULL.
            if (limoi_port->buf_mem_pool)
            {
                LIMDBG("%s reset port %u buffer memory pool.\n",
                       base->name, limoi_port->index);
                lim_mem_pool_reset(limoi_port->buf_mem_pool);
            }
            release_buffers(&limoi_port->buffers_list_head);
        }
    }

    //non-supplier and non-tunneled port must wait till all buffers are freed.
    while (limoi_port->buffers_list_head.next != &limoi_port->buffers_list_head)
    {
        if (lthread_cond_timedwait(&limoi_port->cond,
                                   &limoi_port->mutex, 5000))
        {
            LIMERR("%s port %u free buffer wait timed out\n"
                   "\tpopulated    : %d\n"
                   "\tbuffer count : %d\n",
                   base->name, limoi_port->index,
                   port->bPopulated, port_buffer_num(limoi_port));
            break;
        }
    }
    LIMOI_PORTDEF(limoi_port)->bPopulated = OMX_FALSE;
    lthread_mutex_unlock(&limoi_port->mutex);
    LIMDBG("%s free buffers of port %u done\n",
           base->name, limoi_port->index);

    return 0;
}

OMX_ERRORTYPE port_free_buffer_all (OMX_HANDLETYPE hComp)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    list_t * list;

    // loop through ports to free buffers
    LIMDBG("%s freeing buffers\n", base->name);

    list = base->ports_list_head.prev;
    while (list != &base->ports_list_head)
    {
        lthread_mutex_unlock(&base->mutex);
        if (free_port_buffer(hComp, (limoi_port_t *)list))
        {
            LIMERR("%s unable to free buffer for port: %u.\n",
                   base->name, ((limoi_port_t *)list)->index);
            return OMX_ErrorUndefined;
        }

        lthread_mutex_lock(&base->mutex);
        list = list->prev;
    }

    return OMX_ErrorNone;
}

static int flush_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    OMX_COMPONENTTYPE * comp = (OMX_COMPONENTTYPE *)hComp;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_BUFFERHEADERTYPE ** buf_header = NULL;
    int buf_count = 0;
    int buf_total;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);

    LIMDBG("%s flushing port %u buffer, count: %d.\n",
           base->name, limoi_port->index,
           limq_entries(limoi_port->bufq));

    buf_total = port_buffer_num(limoi_port);
    //component must have been in PAUSED state, otherwise flush
    //may end up in a dead loop under tunneling mode. 'buf_count' is
    //counted up to buf_total to prevent just that.
    while (buf_count < buf_total)
    {
        buf_header = limq_dequeue_get(limoi_port->bufq);
        if (NULL == buf_header)
        {
            LIMDBG("%s no more buffer to flush for port %u, done? %d\n",
                   base->name, limoi_port->index,
                   limq_entries(limoi_port->bufq));
            //limq_entries may not be zero in buffer sharing case, where
            //buffer share requests are flushed by tunnel port.
            //ASSERT(limq_entries(limoi_port->bufq) == 0);
            break;
        }

        //put back buffer to queue before notifying the tunnel port
        //to prevent buffer queue overflow. It is entirely
        //possible that tunnel gets the buffer and immediately sends
        //it back before port gets a chance to put the buffer back.
        if (limq_dequeue_put(limoi_port->bufq, buf_header))
        {
            //buffer is still busy, component has transitioned out of
            //pause state before waiting flush to complete?
            LIMWARN("BUG: %s port %u buffer busy, flushing bailed at state: %s.\n",
                    base->name, limoi_port->index,
                    limoi_omx_state_string(base->state));
            break;
        }

        (*buf_header)->nOffset = 0;
        (*buf_header)->nFilledLen = 0;
        lthread_mutex_unlock(&limoi_port->mutex);

        if (limoi_port->tunnel_comp)
        {
            if (port->eDir == OMX_DirOutput)
            {
                LIMDBG("%s port %u asking tunnel port %u to empty buffer.\n",
                       base->name, limoi_port->index,
                       limoi_port->tunnel_port);
                OMX_EmptyThisBuffer
                    (limoi_port->tunnel_comp, *buf_header);
            } else
            {
                LIMDBG("%s port %u asking tunnel port %u to fill buffer.\n",
                       base->name, limoi_port->index,
                       limoi_port->tunnel_port);
                OMX_FillThisBuffer
                    (limoi_port->tunnel_comp, *buf_header);
            }
        } else
        {
            if (port->eDir == OMX_DirOutput
                && base->callbacks.FillBufferDone)
            {
                base->callbacks.FillBufferDone
                    (hComp, comp->pApplicationPrivate, *buf_header);
            } else if (port->eDir == OMX_DirInput
                       && base->callbacks.EmptyBufferDone)
            {
                base->callbacks.EmptyBufferDone
                    (hComp, comp->pApplicationPrivate, *buf_header);
            }
        }
        lthread_mutex_lock(&limoi_port->mutex);
        buf_count++;
    }

    LIMDBG("%s flushing port %u buffer done, count: %d.\n",
           base->name, limoi_port->index, limq_entries(limoi_port->bufq));

    lthread_cond_broadcast(&limoi_port->cond);
    lthread_mutex_unlock(&limoi_port->mutex);
    return ret;
}

//remove buffers from bufq.
//it is guaranteed that either port->bEnabled is false, or bufq is in
//state transition, in other words, no buffer can come in unless this
//is a supplier port.
static int deactivate_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port)
{
    OMX_COMPONENTTYPE * comp = (OMX_COMPONENTTYPE *)hComp;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_PARAM_BUFFERSUPPLIERTYPE * supplier;
    OMX_BUFFERHEADERTYPE ** buf_header = NULL;
    int buf_count = 0;
    int buf_total;

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);

    buf_total = port_buffer_num(limoi_port);
    LIMDBG("%s deactivating port %u buffer, %d.\n",
           base->name, limoi_port->index, buf_total);

    while (buf_count < buf_total)
    {
        buf_header = limq_dequeue_get(limoi_port->bufq);
        if (NULL == buf_header)
        {
            if (!limoi_port->tunnel_comp)
            {
                LIMDBG("%s no more buffer to deactivate for port %u\n",
                       base->name, limoi_port->index);
                break;
            } else if (((supplier->eBufferSupplier == OMX_BufferSupplyInput
                         && port->eDir == OMX_DirInput)
                        || (supplier->eBufferSupplier == OMX_BufferSupplyOutput
                            && port->eDir == OMX_DirOutput))
                       && limoi_port->activated)
            {
                LIMDBG("%s wait for buffer back from tunnel, count %d/%u.\n",
                       base->name, buf_count, port->nBufferCountActual);

                //we are here because at the time of deactivating this
                //component, the non-supplier tunneled component still
                //owns the supplied buffer, wait here till those buffers
                //are returned.
                if (lthread_cond_timedwait(&limoi_port->cond,
                                           &limoi_port->mutex, 5000))
                {
                    //timeout, if this not due to slow response of down stream
                    //component, it must be that the downstream is not in
                    //the right state transition.
                    LIMERR("%s port %u bufq wait timed out, deadlock?\n"
                           "\tbuf_count : %d\n"
                           "\ttotal num : %d\n",
                           base->name, limoi_port->index,
                           buf_count, port_buffer_num(limoi_port));
                    break;
                } else continue;
            } else
            {
                LIMDBG("%s no more buffer to deactivate for tunnel port %u\n",
                       base->name, limoi_port->index);
                break;
            }
        } else if (!limq_dequeue_put(limoi_port->bufq, buf_header))
        {
            buf_count++;

            //returning buffer to client or its tunneled counter part.
            (*buf_header)->nOffset = 0;
            (*buf_header)->nFilledLen = 0;
            if (limoi_port->tunnel_comp)
            {
                if (supplier->eBufferSupplier == OMX_BufferSupplyInput
                    && port->eDir == OMX_DirOutput)
                {
                    LIMDBG("%s port %u asks tunnel port %u to empty buffer.\n",
                           base->name, limoi_port->index,
                           limoi_port->tunnel_port);
                    lthread_mutex_unlock(&limoi_port->mutex);
                    OMX_EmptyThisBuffer(limoi_port->tunnel_comp, *buf_header);
                    lthread_mutex_lock(&limoi_port->mutex);
                } else if (supplier->eBufferSupplier == OMX_BufferSupplyOutput
                           && port->eDir == OMX_DirInput)
                {
                    LIMDBG("%s port %u asks tunnel port %u to fill buffer.\n",
                           base->name, limoi_port->index,
                           limoi_port->tunnel_port);
                    lthread_mutex_unlock(&limoi_port->mutex);
                    OMX_FillThisBuffer(limoi_port->tunnel_comp, *buf_header);
                    lthread_mutex_lock(&limoi_port->mutex);
                }
            } else
            {
                if (port->eDir == OMX_DirOutput)
                {
                    if (base->callbacks.FillBufferDone)
                    {
                        lthread_mutex_unlock(&limoi_port->mutex);
                        base->callbacks.FillBufferDone
                            (hComp, comp->pApplicationPrivate, *buf_header);
                        lthread_mutex_lock(&limoi_port->mutex);
                    }
                } else
                {
                    if (base->callbacks.EmptyBufferDone)
                    {
                        lthread_mutex_unlock(&limoi_port->mutex);
                        base->callbacks.EmptyBufferDone
                            (hComp, comp->pApplicationPrivate, *buf_header);
                        lthread_mutex_lock(&limoi_port->mutex);
                    }
                }
            }
        } else
        {
            //should never come here.
            LIMWARN("BUG: %s port %u buffer busy upon deactivation.\n",
                    base->name, limoi_port->index);

            lthread_mutex_lock(&base->mutex);
            lthread_cond_broadcast(&base->cond);
            lthread_mutex_unlock(&base->mutex);
            usleep(5000);
        }

        lthread_cond_broadcast(&limoi_port->cond);
    }

    //not really required, but serve as the last resort to
    //make sure no more buffers are held in queue.
    //
    //NO: reseting here may cause buffers permanently lost if,
    // supplier port has pushed back some of its buffers while
    // non-supplier is in the middle of deactivating.
    //
    //limq_reset(limoi_port->bufq);
    //lthread_cond_broadcast(&limoi_port->cond);
    limoi_port->activated = 0;
    lthread_mutex_unlock(&limoi_port->mutex);
    return 0;
}

OMX_ERRORTYPE port_deactivate_buffer_all (OMX_HANDLETYPE hComp)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    list_t * list;

    // loop through ports to deactivate tunnel buffers
    LIMDBG("%s deactivating tunnel buffers\n", base->name);
    list = base->ports_list_head.prev;
    while (list != &base->ports_list_head)
    {
        lthread_mutex_unlock(&base->mutex);
        if (deactivate_port_buffer(hComp, (limoi_port_t *)list))
        {
            lthread_mutex_lock(&base->mutex);
            LIMERR("%s unable to deactivate buffer for port: %u.\n",
                   base->name, ((limoi_port_t *)list)->index);
            return OMX_ErrorUndefined;
        }
        lthread_mutex_lock(&base->mutex);
        list = list->prev;
    }
    return OMX_ErrorNone;
}

void port_disable (limoi_component_cmd_t * cmd)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(cmd->hComponent);
    list_t * list;
    limoi_port_t * limoi_port;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    int event_delivered = 0;
    limoi_component_cmd_t cmd_copy = *cmd;

    LIMDBG("%s disabling port %u\n", base->name, cmd->nParam1);
    lthread_mutex_lock(&base->mutex);
    while (base->busy)
    {
        if (lthread_cond_timedwait(&base->cond, &base->mutex, 5000))
        {
            LIMWARN("%s data processing busy waiting timeout: %x.\n",
                   base->name, base->busy);
            break;
        }
    }

    list = base->ports_list_head.prev;
    while (list != &base->ports_list_head)
    {
        limoi_port = (limoi_port_t *)list;
        if (cmd->nParam1 == limoi_port->index || cmd->nParam1 == OMX_ALL)
        {
            //make sure port->bEnabled is false, in case port disable
            //request is immediately following a port enable request
            OMX_PARAM_PORTDEFINITIONTYPE * port;

            lthread_mutex_lock(&limoi_port->mutex);
            port = LIMOI_PORTDEF(limoi_port);
            port->bEnabled = OMX_FALSE;
            cmd_copy.nParam1 = limoi_port->index;
            lthread_mutex_unlock(&limoi_port->mutex);
            lthread_mutex_unlock(&base->mutex);

            ret = command_callback(base, &cmd_copy, 1);
            if (OMX_ErrorNone != ret)
            {
                LIMERR("%s disable port prep callback returned error %s\n",
                       base->name, limoi_omx_error_string(ret));
            } else
            {
                if (base->state == OMX_StateExecuting
                    || base->state == OMX_StatePause)
                {
                    deactivate_port_buffer(cmd->hComponent, limoi_port);
                }
                if (base->state != OMX_StateLoaded
                    && base->state != OMX_StateInvalid)
                {
                    free_port_buffer(cmd->hComponent, limoi_port);
                }
            }

            if (OMX_ErrorNone == ret)
            {
                ret = command_callback(base, &cmd_copy, 0);
                if (OMX_ErrorNone != ret)
                {
                    LIMERR("%s disable port post callback returned error %s\n",
                           base->name, limoi_omx_error_string(ret));
                }
            }

            if (OMX_ErrorNone == ret)
            {
                event_deliver(cmd->hComponent, OMX_EventCmdComplete,
                              cmd->Cmd, limoi_port->index, NULL);
            } else
            {
                LIMWARN("%s port %u disable error %x.\n",
                        base->name, cmd->nParam1, ret);
                event_deliver(cmd->hComponent, OMX_EventError,
                              ret, 0, NULL);
            }
            event_delivered++;
            lthread_mutex_lock(&base->mutex);

            // disable ALL ports?
            if (cmd->nParam1 == limoi_port->index) break;
        }
        list = list->prev;
    }
    lthread_mutex_unlock(&base->mutex);

    if (!event_delivered)
    {
        LIMWARN("%s port %u no longer exists.\n", base->name, cmd->nParam1);
        event_deliver(cmd->hComponent, OMX_EventError,
                      OMX_ErrorBadPortIndex, 0, NULL);
    }
    LIMDBG("%s disabling port %u done\n", base->name, cmd->nParam1);
}

void port_enable (limoi_component_cmd_t * cmd)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(cmd->hComponent);
    list_t * list = base->ports_list_head.next;
    limoi_port_t * limoi_port;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_STATETYPE cur_state;
    int event_delivered = 0;
    limoi_component_cmd_t cmd_copy = *cmd;

    LIMDBG("%s enabling port %u\n", base->name, cmd->nParam1);
    lthread_mutex_lock(&base->mutex);
    list = base->ports_list_head.next;
    while (list != &base->ports_list_head)
    {
        limoi_port = (limoi_port_t *)list;

        if (cmd->nParam1 == limoi_port->index || cmd->nParam1 == OMX_ALL)
        {
            cur_state = base->state;
            //note that state transition is synchronous, thus it is
            //safe to relinquish base->mutex here.
            //plus, allocating port buffer may require to call API of
            //tunneled component, all mutex must be relinquished before
            //doing that.
            lthread_mutex_unlock(&base->mutex);

            cmd_copy.nParam1 = limoi_port->index;
            ret = command_callback(base, &cmd_copy, 1);
            if (OMX_ErrorNone != ret)
            {
                LIMERR("%s enable port prep callback returned error %s\n",
                       base->name, limoi_omx_error_string(ret));
            } else
            {
                lthread_mutex_lock(&limoi_port->mutex);
                LIMOI_PORTDEF(limoi_port)->bEnabled = OMX_TRUE;
                lthread_mutex_unlock(&limoi_port->mutex);

                if (OMX_StateLoaded == cur_state)
                {
                    //do nothing
                } else if (OMX_StateIdle == cur_state)
                {
                    if (port_allocate_buffer(cmd->hComponent, limoi_port))
                    {
                        LIMERR("%s unable to allocate port %u buffer.\n",
                               base->name, limoi_port->index);
                        ret = OMX_ErrorInsufficientResources;
                    }
                } else if (OMX_StateExecuting == cur_state ||
                           OMX_StatePause == cur_state)
                {
                    if (port_allocate_buffer(cmd->hComponent, limoi_port))
                    {
                        LIMERR("%s unable to allocate port %u buffer.\n",
                               base->name, limoi_port->index);
                        ret = OMX_ErrorInsufficientResources;
                    } else if (activate_port_buffer
                               (cmd->hComponent, limoi_port))
                    {
                        LIMERR("%s unable to activate port %u buffer.\n",
                               base->name, limoi_port->index);
                        ret = OMX_ErrorPortUnresponsiveDuringAllocation;
                    }
                }
            }

            if (OMX_ErrorNone == ret)
            {
                ret = command_callback(base, &cmd_copy, 0);
                if (OMX_ErrorNone != ret)
                {
                    LIMERR("%s enable port post callback returned error %s\n",
                           base->name, limoi_omx_error_string(ret));
                }
            }

            if (OMX_ErrorNone == ret)
            {
                event_deliver(cmd->hComponent, OMX_EventCmdComplete,
                              cmd->Cmd, limoi_port->index, NULL);
            } else
            {
                lthread_mutex_lock(&limoi_port->mutex);
                LIMOI_PORTDEF(limoi_port)->bEnabled = OMX_FALSE;
                lthread_mutex_unlock(&limoi_port->mutex);

                event_deliver(cmd->hComponent, OMX_EventError,
                              ret, 0, NULL);
            }
            lthread_mutex_lock(&base->mutex);
            event_delivered++;
            // enable ALL ports?
            if (cmd->nParam1 == limoi_port->index) break;
        }
        list = list->next;
    }
    //broadcast port state change in case main thread is blocking
    //because no port has been enabled.
    lthread_cond_broadcast(&base->cond);
    lthread_mutex_unlock(&base->mutex);
    if (!event_delivered)
    {
        LIMWARN("%s port %u no longer exists.\n", base->name, cmd->nParam1);
        event_deliver(cmd->hComponent, OMX_EventError,
                      OMX_ErrorBadPortIndex, 0, NULL);
    }
    LIMDBG("%s enabling port %u done\n", base->name, cmd->nParam1);
}

void port_flush (limoi_component_cmd_t * cmd)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(cmd->hComponent);
    list_t * list;
    limoi_port_t * limoi_port;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    int event_delivered = 0;
    limoi_component_cmd_t cmd_copy = *cmd;

    LIMDBG("%s flushing port %u\n", base->name, cmd->nParam1);
    lthread_mutex_lock(&base->mutex);
    base->state_transition = 1;
    lthread_cond_broadcast(&base->cond);

    //flushing during active data processing is not allowed.
    while (base->busy)
    {
        if (lthread_cond_timedwait(&base->cond, &base->mutex, 5000))
        {
            LIMWARN("%s data processing busy waiting timeout:\n"
                    "\t busy flag     : %x\n"
                    "\t current state : %s\n",
                    base->name, base->busy,
                    limoi_omx_state_string(base->state));
            break;
        }
    }

    list = base->ports_list_head.next;
    while (list != &base->ports_list_head)
    {
        limoi_port = (limoi_port_t *)list;
        if (cmd->nParam1 == limoi_port->index || cmd->nParam1 == OMX_ALL)
        {
            lthread_mutex_unlock(&base->mutex);

            cmd_copy.nParam1 = limoi_port->index;
            ret = command_callback(base, &cmd_copy, 1);
            if (OMX_ErrorNone != ret)
            {
                LIMERR("%s flush port prep callback returned error %s\n",
                       base->name, limoi_omx_error_string(ret));
            } else
            {
                ret = flush_port_buffer(cmd->hComponent, limoi_port);
            }

            if (OMX_ErrorNone == ret)
            {
                ret = command_callback(base, &cmd_copy, 0);
                if (OMX_ErrorNone != ret)
                {
                    LIMERR("%s flush port post callback returned error %s\n",
                           base->name, limoi_omx_error_string(ret));
                }
            }

            if (OMX_ErrorNone == ret)
            {
                event_deliver(cmd->hComponent, OMX_EventCmdComplete,
                              cmd->Cmd, limoi_port->index, NULL);
            } else
            {
                LIMWARN("%s port %u flush error %x.\n",
                        base->name, cmd->nParam1, ret);
                event_deliver(cmd->hComponent, OMX_EventError,
                              ret, 0, NULL);
            }
            lthread_mutex_lock(&base->mutex);

            event_delivered++;
            // flush ALL ports?
            if (cmd->nParam1 == limoi_port->index) break;
        }
        list = list->next;
    }
    base->state_transition = 0;
    lthread_cond_broadcast(&base->cond);
    lthread_mutex_unlock(&base->mutex);

    if (!event_delivered)
    {
        LIMWARN("%s port %u no longer exists.\n", base->name, cmd->nParam1);
        event_deliver(cmd->hComponent, OMX_EventError,
                      OMX_ErrorBadPortIndex, 0, NULL);
    }
    LIMDBG("%s flushing port %u done\n", base->name, cmd->nParam1);
}

void port_disable_now (
    OMX_HANDLETYPE hComp,
    OMX_U32 nPortIndex)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    list_t * list;
    limoi_port_t * limoi_port;

    LIMDBG("%s disabling port %u now.\n", base->name, nPortIndex);
    list = base->ports_list_head.next;
    while (list != &base->ports_list_head)
    {
        limoi_port = (limoi_port_t *)list;
        lthread_mutex_lock(&limoi_port->mutex);
        if (nPortIndex == limoi_port->index || nPortIndex == OMX_ALL)
        {
            LIMOI_PORTDEF(limoi_port)->bEnabled = OMX_FALSE;
            lthread_cond_broadcast(&limoi_port->cond);
        }
        if (nPortIndex == limoi_port->index)
        {
            lthread_mutex_unlock(&limoi_port->mutex);
            break;
        }
        lthread_mutex_unlock(&limoi_port->mutex);
        list = list->next;
    }
}

OMX_ERRORTYPE omx_use_buffer (
    OMX_HANDLETYPE hComponent,
    OMX_BUFFERHEADERTYPE** ppBufferHdr,
    OMX_U32 nPortIndex,
    OMX_PTR pAppPrivate,
    OMX_U32 nSizeBytes,
    OMX_U8* pBuffer)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_port_t * limoi_port;

    LIMDBG("%s port %u uses buffer %p\n",
           LIMOI_COMPONENT_PRIV(hComponent)->name, nPortIndex, pBuffer);

    limoi_port = limoi_component_get_port(hComponent, nPortIndex);
    if (NULL == limoi_port)
    {
        ret = OMX_ErrorBadParameter;
        goto bail;
    }
    ret = use_buffer(hComponent, limoi_port, ppBufferHdr,
                     pAppPrivate, nSizeBytes, pBuffer);

 bail:
    return ret;
}

OMX_ERRORTYPE omx_allocate_buffer (
    OMX_HANDLETYPE hComponent,
    OMX_BUFFERHEADERTYPE** ppBuffer,
    OMX_U32 nPortIndex,
    OMX_PTR pAppPrivate,
    OMX_U32 nSizeBytes)
{
    return omx_use_buffer(hComponent, ppBuffer, nPortIndex,
                          pAppPrivate, nSizeBytes, NULL);
}

OMX_ERRORTYPE omx_free_buffer (
    OMX_HANDLETYPE hComponent,
    OMX_U32 nPortIndex,
    OMX_BUFFERHEADERTYPE* pBuffer)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_port_buffer_t * port_buf;
    limoi_port_t * limoi_port;
    list_t * list;

    limoi_port = limoi_component_get_port(hComponent, nPortIndex);
    if (NULL == limoi_port)
    {
        LIMERR("%s unable to find port: %u\n", base->name, nPortIndex);
        ret = OMX_ErrorBadParameter;
        goto bail;
    }

    lthread_mutex_lock(&limoi_port->mutex);

    list = limoi_port->buffers_list_head.next;
    while (list != &limoi_port->buffers_list_head)
    {
        port_buf = (limoi_port_buffer_t *)list;
        LIMDBG("%s port %u free buffer: %p | %p\n",
               base->name, nPortIndex,
               pBuffer->pBuffer, port_buf->header->pBuffer);

        list = list->next;
        if (port_buf->header->pBuffer == pBuffer->pBuffer)
        {
            LIMDBG("%s port %u free buffer done: %p | %p\n",
                   base->name, nPortIndex, pBuffer, pBuffer->pBuffer);
            if (port_buf->allocated)
            {
                ASSERT(limoi_port->buf_mem_pool);
                lim_mem_pool_put_buf
                    (limoi_port->buf_mem_pool, pBuffer->pBuffer);
            }
            list_del(&port_buf->list);
            free(port_buf);

            //broadcast new port condition.
            lthread_cond_broadcast(&limoi_port->cond);
            goto bail2;
        }
    }
    LIMWARN("%s unable to find buffer %p : %p on port %u\n",
            base->name, pBuffer, pBuffer->pBuffer, nPortIndex);
    //LIMOI_PORTDEF(limoi_port)->bPopulated = OMX_FALSE;
    ret = OMX_ErrorBadParameter;

 bail2:
    lthread_mutex_unlock(&limoi_port->mutex);

 bail:
    return ret;
}

OMX_ERRORTYPE omx_empty_this_buffer (
    OMX_HANDLETYPE hComponent,
    OMX_BUFFERHEADERTYPE* pBuffer)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_port_t * limoi_port;
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_PARAM_BUFFERSUPPLIERTYPE * supplier;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);

    if (pBuffer->nInputPortIndex == 0x7fffffff)
    {
        LIMERR("%s EmptyThisBuffer called on output port?\n", base->name);
        return OMX_ErrorBadPortIndex;
    }

    LIMDBG("%s port %d emptying this buffer len: %u.\n",
           base->name, pBuffer->nInputPortIndex, pBuffer->nFilledLen);

    limoi_port = limoi_component_get_port
        (hComponent, pBuffer->nInputPortIndex);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);
    if (port->bPopulated == OMX_FALSE)
    {
        LIMDBG("%s invalid state %d, port %u\n"
               "\tenabled     : %d\n"
               "\tpopulated   : %d\n"
               "\ttrasnistion : %d\n",
               base->name, base->state, limoi_port->index,
               port->bEnabled, port->bPopulated,
               limoi_port->bufq_state_transition);
        ret = OMX_ErrorInvalidState;
        goto bail;
    }

    ret = enqueue_this_buffer(hComponent, limoi_port, pBuffer);
    lthread_cond_broadcast(&limoi_port->cond);

    LIMDBG("%s empty buffer of port %u:\n"
           "\taddr          : %p\n"
           "\tfilled length : %u\n"
           "\tcurrent count : %d\n"
           "\ttotal         : %u\n",
           base->name, pBuffer->nInputPortIndex,
           pBuffer->pBuffer, pBuffer->nFilledLen,
           limq_entries(limoi_port->bufq), port->nBufferCountActual);

    ASSERT(limq_entries(limoi_port->bufq) <= port->nBufferCountActual);

 bail:
    lthread_mutex_unlock(&limoi_port->mutex);

    if (OMX_ErrorNone == ret && base->synchronous
        && limq_entries(limoi_port->bufq) >= port->nBufferCountActual / 2)
        sync_consume_buffers(hComponent);

    return ret;
}

OMX_ERRORTYPE omx_fill_this_buffer (
    OMX_HANDLETYPE hComponent,
    OMX_BUFFERHEADERTYPE* pBuffer)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_port_t * limoi_port;
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    OMX_PARAM_BUFFERSUPPLIERTYPE * supplier;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);

    if (pBuffer->nOutputPortIndex == 0x7fffffff)
    {
        LIMERR("%s FillThisBuffer called on input port?\n", base->name);
        return OMX_ErrorBadPortIndex;
    }

    LIMDBG("%s port %u filling this buffer: %p.\n",
           base->name, pBuffer->nOutputPortIndex, pBuffer);

    limoi_port = limoi_component_get_port
        (hComponent, pBuffer->nOutputPortIndex);
    ASSERT(limoi_port);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);
    if (port->bPopulated == OMX_FALSE)
    {
        LIMDBG("%s invalid state %d, port %u\n"
               "\tenabled     : %d\n"
               "\tpopulated   : %d\n"
               "\ttrasnistion : %d\n",
               base->name, base->state, limoi_port->index,
               port->bEnabled, port->bPopulated,
               limoi_port->bufq_state_transition);
        lthread_mutex_unlock(&limoi_port->mutex);
        return OMX_ErrorInvalidState;
    }

    ret = enqueue_this_buffer(hComponent, limoi_port, pBuffer);
    lthread_cond_broadcast(&limoi_port->cond);

    LIMDBG("%s filled buffer of port %u:\n"
           "\taddr | bytes  : %p | %d\n"
           "\tcurrent count : %d\n"
           "\ttotal         : %u\n",
           base->name, pBuffer->nOutputPortIndex, pBuffer, pBuffer->nFilledLen,
           limq_entries(limoi_port->bufq), port->nBufferCountActual);

    ASSERT(limq_entries(limoi_port->bufq) <= port->nBufferCountActual);

    lthread_mutex_unlock(&limoi_port->mutex);

    lthread_mutex_lock(&base->mutex);
    if (base->buffer_share_policy && base->buffer_share_pool)
        buffer_share_pool_out(hComponent, limoi_port, pBuffer);
    lthread_mutex_unlock(&base->mutex);

    if (OMX_ErrorNone == ret && base->synchronous
        && limq_entries(limoi_port->bufq) >= port->nBufferCountActual / 2)
        sync_consume_buffers(hComponent);

    return ret;
}

int limoi_component_add_port (
    OMX_HANDLETYPE hComp,
    const OMX_PARAM_PORTDEFINITIONTYPE * port)
{
    limoi_port_t * reg;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    OMX_PARAM_BUFFERSUPPLIERTYPE supplier;
    char bufq_name[LIMQ_NAME_MAX_BYTES];

    if (NULL == port) return -1;

    reg = malloc(sizeof(limoi_port_t));
    if (NULL == reg) return -1;

    LIMDBG("adding %s port %u to %s\n",
           (port->eDir == OMX_DirInput)? "input" : "output",
           port->nPortIndex, base->name);

    reg->buf_mem_pool = NULL;
    reg->index = port->nPortIndex;
    list_init(&reg->params_list_head);
    list_init(&reg->buffers_list_head);

    snprintf(bufq_name, LIMQ_NAME_MAX_BYTES, "%s port %u buffer queue",
             base->name, port->nPortIndex);
    reg->bufq = limq_create(sizeof(void *), LIMOI_COMPONENT_PORT_BUF_Q_MAX,
                            bufq_name);
    if (NULL == reg->bufq)
    {
        LIMERR("out of memory.\n");
        free(reg);
        return -1;
    }

    //WARNING: do NOT change the order of the following code.
    //         there are macros relied on how parameters are added.
    if (param_allocate_and_set(OMX_IndexParamPortDefinition, (void *)port,
                               sizeof(OMX_PARAM_PORTDEFINITIONTYPE),
                               &reg->params_list_head))
    {
        LIMERR("out of memory.\n");
        limq_destroy(reg->bufq);
        free(reg);
        return -1;
    }

    limoi_set_spec_version(&supplier, sizeof(OMX_PARAM_BUFFERSUPPLIERTYPE));
    supplier.nPortIndex = port->nPortIndex;
    supplier.eBufferSupplier = OMX_BufferSupplyUnspecified;
    if (param_allocate_and_set(OMX_IndexParamCompBufferSupplier, &supplier,
                               sizeof(OMX_PARAM_BUFFERSUPPLIERTYPE),
                               &reg->params_list_head))
    {
        LIMERR("out of memory.\n");
        param_release(&reg->params_list_head);
        limq_destroy(reg->bufq);
        free(reg);
        return -1;
    }

    lthread_mutex_init2(&reg->mutex, LTHREAD_MUTEX_RECURSIVE);
    lthread_cond_init(&reg->cond);

    //finalizing limoi_port initialization
    reg->buffer_share.type = LIMOX_BufferShareDisabled;
    reg->buffer_share.share_num = 0;
    reg->tunnel_comp = NULL;
    reg->tunnel_port = -1;
    reg->bufq_state_transition = 0;
	list_init(&reg->list);

    lthread_mutex_lock(&base->mutex);
	list_add(&reg->list, &base->ports_list_head);
    lthread_mutex_unlock(&base->mutex);

    return 0;
}

limoi_port_t * limoi_component_get_port (
    OMX_HANDLETYPE hComponent, OMX_U32 nPortIndex)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);
    list_t * port;

    //LIMDBG("%s get port %u.\n", base->name, nPortIndex);
    lthread_mutex_lock(&base->mutex);
    port = base->ports_list_head.next;
    while (port != &base->ports_list_head)
    {
        if (nPortIndex == ((limoi_port_t *)port)->index)
        {
            lthread_mutex_unlock(&base->mutex);
            //LIMDBG("%s get port %u done.\n", base->name, nPortIndex);
            return (limoi_port_t *)port;
        }
        port = port->next;
    }
    lthread_mutex_unlock(&base->mutex);
    LIMERR("%s unable to get port %u.\n", base->name, nPortIndex);
    return NULL;
}

OMX_BUFFERHEADERTYPE ** limoi_component_get_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    int timeout)
{
    OMX_BUFFERHEADERTYPE ** buf_header = NULL;
    OMX_PARAM_PORTDEFINITIONTYPE * port;

    LIMDBG("%s get port %u buffer.\n",
           LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);

    while (1)
    {
        if (port->bEnabled == OMX_TRUE && port->bPopulated == OMX_TRUE
            && !limoi_port->bufq_state_transition)
        {
            buf_header = limq_dequeue_get(limoi_port->bufq);

            if (buf_header)
            {
                ASSERT((*buf_header)->pBuffer);
                LIMDBG("%s port %u buffer fetched as %p %p %u/%u\n"
                       "\tcount  : %d\n"
                       "\ttotal  : %d\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name,
                       limoi_port->index,
                       *buf_header,
                       (*buf_header)->pBuffer,
                       (*buf_header)->nFilledLen,
                       (*buf_header)->nAllocLen,
                       limq_entries(limoi_port->bufq),
                       port->nBufferCountActual);
                lthread_mutex_unlock(&limoi_port->mutex);
                return buf_header;
            } else if (LIMOI_COMPONENT_PRIV(hComp)->synchronous)
            {
                LIMDBG("%s port %u buffer not available:\n"
                       "\tbEnabled                 : %d\n"
                       "\tbPopulated               : %d\n"
                       "\tbufq_transition_state    : %d\n"
                       "\ttotal                    : %u\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name,
                       limoi_port->index,
                       port->bEnabled, port->bPopulated,
                       limoi_port->bufq_state_transition,
                       port->nBufferCountActual);
                break;
            }
        } else
        {
            LIMDBG("%s port %u not enabled or in state transition.\n"
                   "\tbEnabled                 : %d\n"
                   "\tbPopulated               : %d\n"
                   "\tbufq_transition_state    : %d\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name,
                   limoi_port->index,
                   port->bEnabled, port->bPopulated,
                   limoi_port->bufq_state_transition);
            break;
        }

        //no buffer available, wait if component is asynchronous.
        LIMDBG("%s get port %u buffer waiting on %p %p.\n"
               "\tbEnabled                 : %d\n"
               "\tbPopulated               : %d\n"
               "\tbufq_transition_state    : %d\n"
               "\tbufq entries             : %d\n",
               LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index,
               &limoi_port->cond, &limoi_port->mutex,
               port->bEnabled, port->bPopulated,
               limoi_port->bufq_state_transition,
               limq_entries(limoi_port->bufq));
        if (lthread_cond_timedwait(&limoi_port->cond,
                                   &limoi_port->mutex, timeout))
        {
            if (timeout)
                LIMERR("%s port %u buffer queue wait timed out.\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index);
            break;
        }
    }
    lthread_mutex_unlock(&limoi_port->mutex);
    return NULL;
}

int limoi_component_realloc_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE * buf_header,
    int bytes)
{
    int ret = -1;
    limoi_port_t * tunnel_port;
    OMX_PARAM_BUFFERSUPPLIERTYPE *supplier;
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    void * old_buf;

    ASSERT(limoi_port && buf_header);
    if (!limoi_port->tunnel_comp)
    {
        LIMWARN("%s port %u buffer reallocation only supported on tunnel.\n",
                LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index);
        goto bail;
    }

    if (limoi_port->buffer_share.type != LIMOX_BufferShareDisabled)
    {
        //the reason is obvious, or you think it is not complicated enough?
        LIMWARN("%s port %u reallocation not supported on buffer share.\n",
                LIMOI_COMPONENT_PRIV(hComp)->name);
        goto bail;
    }

    tunnel_port = limoi_component_get_port
        (limoi_port->tunnel_comp, limoi_port->tunnel_port);
    if (NULL == tunnel_port)
    {
        LIMERR("%s unable to get tunnel port: %u %u\n",
               LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index,
               limoi_port->tunnel_port);
        goto bail;
    }

    old_buf = buf_header->pBuffer;

    port = LIMOI_PORTDEF(limoi_port);
    supplier = LIMOI_BUF_SUPPLIER(limoi_port);

    if ((supplier->eBufferSupplier == OMX_BufferSupplyInput
         && port->eDir == OMX_DirInput)
        || (supplier->eBufferSupplier == OMX_BufferSupplyOutput
            && port->eDir == OMX_DirOutput))
    {
        lthread_mutex_lock(&limoi_port->mutex);
        if (limoi_port->buf_mem_pool)
        {
            //supplier returns buffer to and allocates new
            //one from its own memory pool.
            lim_mem_pool_put_buf
                (limoi_port->buf_mem_pool, buf_header->pBuffer);
            buf_header->pBuffer = lim_mem_pool_get_buf
                (limoi_port->buf_mem_pool, bytes);
            if (NULL == buf_header->pBuffer)
            {
                LIMERR("%s unable to realloc port %u buffer.\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index);
            } else
            {
                buf_header->nAllocLen = bytes;
                ret = 0;
            }
        } else
        {
            LIMERR("%s unexpected error, NULL buffer memory pool.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index);
        }
        lthread_mutex_unlock(&limoi_port->mutex);
    } else
    {
        lthread_mutex_lock(&tunnel_port->mutex);
        if (tunnel_port->buf_mem_pool)
        {
            //non-supplier returns buffer to and allocates new
            //one from its tunnel memory pool.
            lim_mem_pool_put_buf
                (tunnel_port->buf_mem_pool, buf_header->pBuffer);
            buf_header->pBuffer = lim_mem_pool_get_buf
                (tunnel_port->buf_mem_pool, bytes);
            if (NULL == buf_header->pBuffer)
            {
                LIMERR("%s unable to realloc port %u buffer.\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name,
                       tunnel_port->index);
            } else
            {
                buf_header->nAllocLen = bytes;
                ret = 0;
            }
        } else
        {
            LIMERR("%s unexpected error, NULL buffer memory pool.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name, tunnel_port->index);
        }
        lthread_mutex_unlock(&tunnel_port->mutex);
    }

    if (0 == ret)
    {
        list_t * list;
        limoi_port_buffer_t * port_buf;

        //always update tunnel port buffers list.
        lthread_mutex_lock(&tunnel_port->mutex);

        list = tunnel_port->buffers_list_head.next;
        while (list != &tunnel_port->buffers_list_head)
        {
            port_buf = (limoi_port_buffer_t *)list;
            if (port_buf->header->pBuffer == old_buf)
            {
                port_buf->header->pBuffer = buf_header->pBuffer;
                break;
            } else list = list->next;
        }
        ASSERT(list != &tunnel_port->buffers_list_head);

        lthread_mutex_unlock(&tunnel_port->mutex);
    }

 bail:
    return ret;
}

int limoi_component_hold_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE ** buf_header)
{
    lthread_mutex_lock(&limoi_port->mutex);
    limq_dequeue_hold(limoi_port->bufq, buf_header);
    lthread_mutex_unlock(&limoi_port->mutex);
    return 0;
}

//caller must own port mutex
static int commit_port_buffer (
    OMX_HANDLETYPE hComp,
    OMX_PARAM_PORTDEFINITIONTYPE * port,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE * buf_header)
{
    OMX_ERRORTYPE res = OMX_ErrorNone;
    int ret = 0;

    //port lock has to be released before calling client or tunneled component
    //API to prevent dead lock
    if (limoi_port->tunnel_comp)
    {
        lthread_mutex_unlock(&limoi_port->mutex);
        if (port->eDir == OMX_DirOutput)
        {
            LIMDBG("%s port %u asks tunnel port %u to empty this buffer: %p.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index,
                   limoi_port->tunnel_port, buf_header);
            res = OMX_EmptyThisBuffer(limoi_port->tunnel_comp, buf_header);
        } else
        {
            LIMDBG("%s port %u asks tunnel port %u to fill this buffer: %p.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name, limoi_port->index,
                   limoi_port->tunnel_port, buf_header);
            buf_header->nOffset = 0;
            buf_header->nFilledLen = 0;
            res = OMX_FillThisBuffer(limoi_port->tunnel_comp, buf_header);
        }
        lthread_mutex_lock(&limoi_port->mutex);
    } else
    {
        OMX_COMPONENTTYPE * comp = (OMX_COMPONENTTYPE *)hComp;
        limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);

        lthread_mutex_unlock(&limoi_port->mutex);
        if (port->eDir == OMX_DirOutput)
        {
            if (base->callbacks.FillBufferDone)
            {
                LIMDBG("%s fill buffer done.\n", base->name);
                res = base->callbacks.FillBufferDone
                    (hComp, comp->pApplicationPrivate, buf_header);
            }
        } else
        {
            if (base->callbacks.EmptyBufferDone)
            {
                LIMDBG("%s empty buffer done.\n", base->name);
                res = base->callbacks.EmptyBufferDone
                    (hComp, comp->pApplicationPrivate, buf_header);
            }
        }
        lthread_mutex_lock(&limoi_port->mutex);
    }

    if (OMX_ErrorNone != res)
    {
        //the client or tunneled component reports error, this has
        //to be handled by the application itself, let's sleep and
        //wait till magic happens.
        LIMWARN("%s client or tunneled component reported error.\n",
                LIMOI_COMPONENT_PRIV(hComp)->name);
        usleep(50000);
        ret = -1;
	}

    return ret;
}

//per IL spec, this routine must be able to return within 5 miliseconds
//thus it is safe to have SendCommand blcok waiting on base->putting_buffer
static int put_release_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE ** buf_header,
    int put)
{
    OMX_PARAM_PORTDEFINITIONTYPE * port;
    int ret = 0;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);

    lthread_mutex_lock(&base->mutex);
    base->putting_buffer = 1;
    if (base->lock_count)
    {
        LIMWARN("%s releasing buffer with mutex: %d, possible deadlock?\n",
                base->name, base->lock_count);
    }
    lthread_mutex_unlock(&base->mutex);

    lthread_mutex_lock(&limoi_port->mutex);
    port = LIMOI_PORTDEF(limoi_port);

    LIMDBG("%s port %d buffer put.\n", base->name, limoi_port->index);

    if (port->bPopulated == OMX_FALSE || port->bEnabled == OMX_FALSE
        || limoi_port->bufq_state_transition)
    {
        LIMDBG("%s port %d depopulated disabled or in state transition:\n"
               "\t bPopulated       : %d\n"
               "\t bEnabled         : %d\n"
               "\t state transition : %d\n",
               base->name, limoi_port->index,
               port->bPopulated, port->bEnabled,
               limoi_port->bufq_state_transition);

        ret = -1;
        if (limq_dequeue_release(limoi_port->bufq, buf_header))
        {
            LIMDBG("%s port %d unable to release buffer.\n",
                   base->name, limoi_port->index);
            goto bail;
        }
    } else
    {
        int res = (put)?
            limq_dequeue_put(limoi_port->bufq, buf_header) :
            limq_dequeue_release(limoi_port->bufq, buf_header);
        if (!res)
        {
            if (commit_port_buffer(hComp, port, limoi_port, *buf_header))
                ret = 1;
        } else
        {
            LIMDBG("%s port %d buffer is still busy.\n",
                   base->name, limoi_port->index);
        }
    }

    if (ret) //requeue_entry
    {
        OMX_BUFFERHEADERTYPE ** buf_header2;
        //on error buffer shall be recycled back to caller port.
        LIMDBG("%s port %u is unable to put back buffer.\n",
               base->name, limoi_port->index);
        buf_header2 = limq_enqueue_get(limoi_port->bufq);
        ASSERT(buf_header2);
        *buf_header2 = *buf_header;
        limq_enqueue_put_ext(limoi_port->bufq, buf_header2, 1);
    }

 bail:
    lthread_cond_broadcast(&limoi_port->cond);
    lthread_mutex_unlock(&limoi_port->mutex);

    lthread_mutex_lock(&base->mutex);

    if (ret && base->buffer_share_policy && base->buffer_share_pool)
    {
        OMX_PARAM_PORTDEFINITIONTYPE * port;
        port = LIMOI_PORTDEF(limoi_port);
        if (port->eDir == OMX_DirOutput)
        {
            LIMDBG("%s port %u getting out of share request pool.\n",
                   base->name, limoi_port->index);
            //if for whatever reason, output buffer is not successfully commit,
            //buffer needs to be removed out of buffer share request pool.
            buffer_share_pool_out(hComp, limoi_port, *buf_header);
        }
    }

    base->putting_buffer = 0;
    lthread_cond_broadcast(&base->cond);
    lthread_mutex_unlock(&base->mutex);

    return ret;
}

int limoi_component_release_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE ** buf_header)
{
    return put_release_port_buffer(hComp, limoi_port, buf_header, 0);
}

int limoi_component_put_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * limoi_port,
    OMX_BUFFERHEADERTYPE ** buf_header)
{
    return put_release_port_buffer(hComp, limoi_port, buf_header, 1);
}

int limoi_component_share_port_buffer (
    OMX_HANDLETYPE hComp,
    limoi_port_t * in_port,
    limoi_port_t * out_port,
    OMX_BUFFERHEADERTYPE ** in_buf_header,
    OMX_BUFFERHEADERTYPE * out_buf_header)
{
    int ret = -1;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);

    if (!base->buffer_share_policy || !base->buffer_share_pool) goto bail;

    //swap pBuffer so that port buffer headers always contain unique buffer.
    if (out_buf_header->pBuffer != (*in_buf_header)->pBuffer)
    {
        list_t * list;

        lthread_mutex_lock(&out_port->mutex);
        list = out_port->buffers_list_head.next;
        while (list != &out_port->buffers_list_head)
        {
            if (((limoi_port_buffer_t *)list)->header->pBuffer ==
                (*in_buf_header)->pBuffer)
            {
                ((limoi_port_buffer_t *)list)->header->pBuffer =
                    out_buf_header->pBuffer;
                out_buf_header->pBuffer = (*in_buf_header)->pBuffer;
                break;
            } else list = list->next;
        }
        lthread_mutex_unlock(&out_port->mutex);
    }

    ret = buffer_share_pool_in
        (hComp, in_port, out_port, in_buf_header, out_buf_header);
    if (!ret)
        limoi_component_hold_port_buffer(hComp, in_port, in_buf_header);

 bail:
    return ret;
}
