/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL component role support subroutines.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-07-22 MG
*/
#define _GNU_SOURCE
#include "limoi.h"
#include "component.h"
#include "string.h"
#include <stdarg.h>

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

void role_release (list_t * roles_list_head)
{
    list_t * list = roles_list_head->next;
    limoi_role_t * reg;

    while (list != roles_list_head)
    {
        reg = (limoi_role_t *)list;

        LIMDBG("release role: %s\n", reg->role_name);

        free((void *)reg->role_name);

        list = list->next;

        list_del(&reg->list);
        free(reg);
    }
}

OMX_ERRORTYPE omx_component_role_enum (
    OMX_HANDLETYPE hComponent,
    OMX_U8 *cRole,
    OMX_U32 nIndex)
{
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComponent);
    list_t * list;
    limoi_role_t * reg;
    int idx = 0;
    OMX_ERRORTYPE ret = OMX_ErrorNoMore;

    if (NULL == cRole) return OMX_ErrorBadParameter;

    lthread_mutex_lock(&base->mutex);
    list = base->roles_list_head.next;
    while (list != &base->roles_list_head)
    {
        reg = (limoi_role_t *)list;
        if (reg->role_name)
        {
            if (idx == nIndex)
            {
                strcpy((char *)cRole, reg->role_name);
                ret = OMX_ErrorNone;
                break;
            } else idx++;
        }
        list = list->next;
    }

    lthread_mutex_unlock(&base->mutex);
    return ret;
}

int limoi_component_add_role (
    OMX_HANDLETYPE hComp,
    const char * role)
{
    list_t * list;
    limoi_role_t * reg;
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);
    int ret = 0;

    if (NULL == role) return -1;

    reg = malloc(sizeof(limoi_role_t));
    if (NULL == reg)
    {
        LIMERR("out of memory.\n");
        return -1;
    }

    LIMDBG("adding role: %s\n", role);

    reg->role_name = strndup(role, OMX_MAX_STRINGNAME_SIZE - 1);
    if (NULL == reg->role_name)
    {
        LIMERR("out of memory.\n");
        free(reg);
        return -1;
    }

    lthread_mutex_lock(&base->mutex);
    list = base->roles_list_head.next;
    while (list != &base->roles_list_head)
    {
        limoi_role_t * role0 = (limoi_role_t *)list;
        int res = strcmp(role0->role_name, reg->role_name);
        if (!res)
        {
            LIMERR("duplicated role name: %s\n", reg->role_name);
            free((void *)reg->role_name);
            free(reg);
            ret = -1;
            goto bail;
        } else if (res > 0) break;
        else list = list->next;
    }

    list_init(&reg->list);
    list_add_tail(&reg->list, list);
 bail:
    lthread_mutex_unlock(&base->mutex);

    return ret;
}

OMX_ERRORTYPE limoi_component_get_current_role (
    OMX_HANDLETYPE hComp,
    char * role)
{
    OMX_PARAM_COMPONENTROLETYPE role_local;
    OMX_ERRORTYPE ret;

    limoi_set_spec_version(&role_local, sizeof(OMX_PARAM_COMPONENTROLETYPE));
    ret = limoi_component_get_parameter
        (hComp, OMX_IndexParamStandardComponentRole,
         &role_local);

    if (OMX_ErrorNone == ret)
    {
        strcpy(role, (char *)role_local.cRole);
    } else
    {
        LIMDBG("%s unable to get current role.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
    }

    return ret;
}
