/*
  Copyright (C) 2010 - 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL standard video scheduler component.

  REVISION:

    1) initial creation. ----------------------------------------- 2010-02-28 MG
*/
#define _GNU_SOURCE
#include "limoi.h"
#include "video-priv.h"

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

//DEBUG: define this to completely bypass the video scheduler, video buffer
//       is immediately scheduled to output (passthru mode) instead.
//#define PASSTHRU_DEBUG


static OMX_ERRORTYPE sub_handle (
    OMX_HANDLETYPE hComp,
    limoi_port_t ** audio_port,
    limoi_port_t ** video_port,
    limoi_port_t ** other_port)
{
    limoi_video_scheduler_t * scheduler = LIMOI_VIDEO_SCHEDULER(hComp);
    limoi_port_t * video_port_in = *video_port;
    limoi_port_t * video_port_out = *(video_port + 1);
    limoi_port_t * clock_port = *other_port;
    OMX_BUFFERHEADERTYPE ** clock_buf;
    OMX_BUFFERHEADERTYPE ** out_buf = NULL;
    OMX_BUFFERHEADERTYPE * buf;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

#ifdef PASSTHRU_DEBUG
    return OMX_ErrorNoMore;
#endif

    if ((!clock_port->tunnel_comp
         && LIMOI_PORTDEF(clock_port)->bEnabled == OMX_TRUE)
        || !scheduler->priv->callbacks.clock)
    {
        ret = OMX_ErrorNoMore;
        goto bail;
    }

    LIMDBG("%s scheduler get clock port buffer.\n",
           LIMOI_COMPONENT_PRIV(hComp)->name);
    clock_buf = limoi_component_get_port_buffer(hComp, clock_port, 0);

    if (NULL == clock_buf)
    {
        LIMDBG("%s scheduler null clock port buffer.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        ret = OMX_ErrorNotReady;
    } else if (!(*clock_buf)->nFilledLen)
    {
        LIMDBG("%s scheduler clock port buffer zero length.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        if (-1 == limoi_component_put_port_buffer(hComp, clock_port, clock_buf))
        {
            ret = OMX_ErrorNotReady;
        }
        clock_buf = NULL;
    }

    LIMDBG("%s scheduler get output port buffer.\n",
           LIMOI_COMPONENT_PRIV(hComp)->name);
    out_buf = limoi_component_get_port_buffer(hComp, video_port_out, 0);
    buf = (out_buf)? *out_buf : NULL;

    if (!(buf && buf->nFilledLen) && clock_buf)
    {
        scheduler->priv->callbacks.clock(hComp, video_port_in, video_port_out,
                                         buf, clock_port->tunnel_comp,
                                         *clock_buf);

        if (!(*clock_buf)->nFilledLen)
        {
            int tmp;

            LIMDBG("%s put back clock buffer.\n", LIMOI_COMPONENT_PRIV(hComp)->name);
            tmp = limoi_component_put_port_buffer(hComp, clock_port, clock_buf);
            if (tmp)
            {
                if (tmp == -1) ret = OMX_ErrorNotReady;
                else
                {
                    LIMDBG("%s unable to put back port buffer.\n",
                           LIMOI_COMPONENT_PRIV(hComp)->name);
                }
            }
            clock_buf = NULL;
        }
    }

    if (buf && (buf->nFilledLen || buf->nFlags & OMX_BUFFERFLAG_EOS))
    {
        int tmp;

        LIMDBG("%s put back output buffer, filled len: %u.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name, buf->nFilledLen);
        tmp = limoi_component_put_port_buffer(hComp, video_port_out, out_buf);
        if (tmp)
        {
            if (tmp == -1) ret = OMX_ErrorNotReady;
            else
            {
                //LIMWARN("%s unable to put back output port buffer.\n",
                //        LIMOI_COMPONENT_PRIV(hComp)->name);
            }
        }
        out_buf = NULL;
    }

    if (clock_buf)
        limoi_component_release_port_buffer(hComp, clock_port, clock_buf);
    if (out_buf)
        limoi_component_release_port_buffer(hComp, video_port_out, out_buf);
 bail:
    return ret;
}

#ifdef PASSTHRU_DEBUG
static int passthru_debug(
    OMX_HANDLETYPE hComp,
    limoi_port_t * in_port,
    limoi_port_t * out_port)
{
    //limoi_video_scheduler_t * scheduler = LIMOI_VIDEO_SCHEDULER(hComp);
    OMX_BUFFERHEADERTYPE ** in;
    OMX_BUFFERHEADERTYPE ** out;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    //process input
    in = limoi_component_get_port_buffer(hComp, in_port, -1);
    if (NULL == in)
    {
        LIMDBG("%s unable to get input port buffer.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        ret = OMX_ErrorNotReady;
    } else if (!(*in)->nFilledLen && !((*in)->nFlags & OMX_BUFFERFLAG_EOS))
    {
        LIMDBG("%s zero length input.\n", LIMOI_COMPONENT_PRIV(hComp)->name);
        limoi_component_put_port_buffer(hComp, in_port, in);
        in = NULL;
    }

    out = limoi_component_get_port_buffer(hComp, out_port, -1);
    if (NULL == out)
    {
        //nothing is wrong with this.
        LIMDBG("%s unable to get output port buffer.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        ret = OMX_ErrorNotReady;
    }

    if (in && out)
    {
        LIMDBG("%s commiting bytes: %d\n", LIMOI_COMPONENT_PRIV(hComp)->name,
               (*in)->nFilledLen);
        if ((*in)->nFilledLen)
        {
            memcpy((*out)->pBuffer, (*in)->pBuffer, (*in)->nFilledLen);
        }
        (*out)->nFilledLen = (*in)->nFilledLen;

        if ((*in)->nFlags & OMX_BUFFERFLAG_EOS)
        {
            (*out)->nFlags |= OMX_BUFFERFLAG_EOS;
            limoi_component_deliver_event
                (hComp, OMX_EventBufferFlag, 0, (*in)->nFlags, NULL);
            limoi_component_deliver_event
                (hComp, OMX_EventBufferFlag, 1, (*out)->nFlags, NULL);
        }

        limoi_component_put_port_buffer(hComp, out_port, out);
        limoi_component_put_port_buffer(hComp, in_port, in);
    } else
    {
        if (in)
        {
            limoi_component_release_port_buffer(hComp, in_port, in);
        }
        if (out)
        {
            limoi_component_release_port_buffer(hComp, out_port, out);
        }
    }

    return ret;
}
#endif

static OMX_ERRORTYPE main_handle (
    OMX_HANDLETYPE hComp,
    limoi_port_t ** audio_port,
    limoi_port_t ** video_port,
    limoi_port_t ** other_port,
    int * preloaded)
{
    limoi_video_scheduler_t * scheduler = LIMOI_VIDEO_SCHEDULER(hComp);
    limoi_port_t * video_port_in = *video_port;
    limoi_port_t * video_port_out = *(video_port + 1);
    limoi_port_t * clock_port_in = *other_port;
    OMX_BUFFERHEADERTYPE ** in_buf;
    OMX_BUFFERHEADERTYPE ** out_buf = NULL;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    if (*preloaded == 0)
    {
        //do not requrie preload.
        *preloaded = LIMOI_PRELOAD_BUFFER_COUNT;
        return OMX_ErrorNone;
    }

#ifdef PASSTHRU_DEBUG
    return passthru_debug(hComp, video_port_in, video_port_out);
#endif

    LIMDBG("%s scheduler get input port buffer.\n",
           LIMOI_COMPONENT_PRIV(hComp)->name);
    in_buf = limoi_component_get_port_buffer(hComp, video_port_in, -1);
    if (NULL == in_buf)
    {
        LIMDBG("%s unable to get port buffer.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        ret = OMX_ErrorNotReady;
        goto bail;
    }
    LIMDBG("%s scheduler get input port buffer done:\n"
           "\t nFilledLen : %u\n"
           "\t buffer     : %p\n",
           LIMOI_COMPONENT_PRIV(hComp)->name, (*in_buf)->nFilledLen, *in_buf);

    if ((*in_buf)->nFilledLen || (*in_buf)->nFlags & OMX_BUFFERFLAG_EOS)
    {
        OMX_BUFFERHEADERTYPE * buf;
        int res = 0;

        LIMDBG("%s scheduler get output port buffer.\n",
               LIMOI_COMPONENT_PRIV(hComp)->name);
        out_buf = limoi_component_get_port_buffer(hComp, video_port_out, 0);
        buf = (out_buf)? *out_buf : NULL;

        if ((*in_buf)->nFilledLen || (*in_buf)->nFlags & OMX_BUFFERFLAG_EOS)
        {
            LIMDBG("%s scheduler input buffer %p bytes: %u.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name,
                   (*in_buf), (*in_buf)->nFilledLen);

            res = scheduler->priv->callbacks.schedule
                (hComp, video_port_in,
                 in_buf, buf,
                 clock_port_in->tunnel_comp,
                 clock_port_in->tunnel_port,
                 clock_port_in->index);
        }

        if ((*in_buf)->nFlags & OMX_BUFFERFLAG_EOS)
        {
            limoi_component_deliver_event
                (hComp, OMX_EventBufferFlag, 0, (*in_buf)->nFlags, NULL);
        }

        if (res == 1)
        {
            LIMDBG("%s release input buffer %p bytes: %u.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name,
                   (*in_buf), (*in_buf)->nFilledLen);
            if (limoi_component_release_port_buffer
                (hComp, video_port_in, in_buf))
            {
                LIMDBG("%s unable to release port buffer.\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name);
            }
            ret = OMX_ErrorNotReady;
        } else
        {
            LIMDBG("%s put back input buffer %p bytes: %u.\n",
                   LIMOI_COMPONENT_PRIV(hComp)->name,
                   (*in_buf), (*in_buf)->nFilledLen);
            if (limoi_component_put_port_buffer(hComp, video_port_in, in_buf))
            {
                LIMDBG("%s unable to put back port buffer.\n",
                       LIMOI_COMPONENT_PRIV(hComp)->name);
            }
        }

        if (buf)
        {
            if (buf->nFilledLen)
            {
                limoi_component_put_port_buffer
                    (hComp, video_port_out, out_buf);
            } else
            {
                limoi_component_release_port_buffer
                    (hComp, video_port_out, out_buf);
            }
        }
    } else
    {
        LIMDBG("%s input buffer has zero length.\n",
                LIMOI_COMPONENT_PRIV(hComp)->name);
        limoi_component_put_port_buffer
            (hComp, video_port_in, in_buf);
    }

 bail:
    return ret;
}

static OMX_ERRORTYPE prep_cmd_handle (limoi_component_cmd_t * cmd)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_video_scheduler_t * scheduler;
    limoi_video_scheduler_priv_t * pscheduler;

    scheduler = LIMOI_VIDEO_SCHEDULER(cmd->hComponent);
    pscheduler = scheduler->priv;

    if (pscheduler->callbacks.command_prep)
    {
        ret = pscheduler->callbacks.command_prep(cmd);
        if (OMX_ErrorNone != ret)
        {
            LIMERR("%s command callback returned error %s\n",
                   LIMOI_COMPONENT_PRIV(cmd->hComponent)->name,
                   limoi_omx_error_string(ret));
        }
    }
    return ret;
}

static OMX_ERRORTYPE post_cmd_handle (limoi_component_cmd_t * cmd)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_video_scheduler_t * scheduler;
    limoi_video_scheduler_priv_t * pscheduler;

    scheduler = LIMOI_VIDEO_SCHEDULER(cmd->hComponent);
    pscheduler = scheduler->priv;

    if (pscheduler->callbacks.command_post)
    {
        ret = pscheduler->callbacks.command_post(cmd);
        if (OMX_ErrorNone != ret)
        {
            LIMERR("%s command callback returned error %s\n",
                   LIMOI_COMPONENT_PRIV(cmd->hComponent)->name,
                   limoi_omx_error_string(ret));
        }
    }
    return ret;
}

static int add_port (OMX_HANDLETYPE hComp)
{
    OMX_PARAM_PORTDEFINITIONTYPE port;
    OMX_PORT_PARAM_TYPE port_param;

    limoi_set_spec_version(&port_param, sizeof(OMX_PORT_PARAM_TYPE));
    port_param.nPorts = 2;
    port_param.nStartPortNumber = 0; //VPB = 0
    limoi_component_set_parameter(hComp, OMX_IndexParamVideoInit, &port_param);

    port_param.nStartPortNumber = 2; //OPB = 2
    port_param.nPorts = 1;
    limoi_component_set_parameter(hComp, OMX_IndexParamOtherInit, &port_param);

    limoi_set_spec_version(&port, sizeof(OMX_PARAM_PORTDEFINITIONTYPE));

    port.nPortIndex = 0;
    port.eDir = OMX_DirInput;
    port.nBufferCountActual = 2;
    port.nBufferCountMin = 2;
    port.nBufferSize = 352 * 288 + 352 * 288 / 2;
    port.bEnabled = OMX_TRUE;
    port.bPopulated = OMX_FALSE;
    port.eDomain = OMX_PortDomainVideo;
    port.bBuffersContiguous = OMX_FALSE;
    port.nBufferAlignment = 0;
    port.format.video.cMIMEType = NULL;
    port.format.video.pNativeRender = 0;
    port.format.video.bFlagErrorConcealment = OMX_FALSE;
    port.format.video.eCompressionFormat = OMX_VIDEO_CodingUnused;
    port.format.video.eColorFormat = OMX_COLOR_FormatYUV420Planar;
    port.format.video.pNativeWindow = NULL;
    limoi_component_add_port(hComp, &port);

    port.nPortIndex = 1;
    port.eDir = OMX_DirOutput;
    limoi_component_add_port(hComp, &port);

    port.nPortIndex = 2;
    port.eDir = OMX_DirInput;
    port.nBufferCountActual = LIMOI_MAX_CLOCK_PORT_BUFFERS;
    port.nBufferCountMin = LIMOI_MAX_CLOCK_PORT_BUFFERS;
    port.nBufferSize = sizeof(OMX_TIME_MEDIATIMETYPE);
    port.eDomain = OMX_PortDomainOther;
    port.format.other.eFormat = OMX_OTHER_FormatTime;
    limoi_component_add_port(hComp, &port);

    return 0;
}

int limoi_video_scheduler_set_callbacks (
    OMX_HANDLETYPE hComp,
    limoi_video_scheduler_callbacks_t * callbacks)
{
    limoi_video_scheduler_t * scheduler = LIMOI_VIDEO_SCHEDULER(hComp);
    limoi_component_priv_t * base = LIMOI_COMPONENT_PRIV(hComp);

    lthread_mutex_lock(&base->mutex);
    scheduler->priv->callbacks = *callbacks;
    lthread_mutex_unlock(&base->mutex);

    return 0;
}

OMX_ERRORTYPE limoi_video_scheduler_deinit (
    OMX_HANDLETYPE hComponent)
{
    limoi_video_scheduler_t * scheduler = LIMOI_VIDEO_SCHEDULER(hComponent);

    free(scheduler->priv);

    // parent destructor.
    return limoi_component_deinit(hComponent);
}

OMX_ERRORTYPE limoi_video_scheduler_init (
    OMX_HANDLETYPE * hComp,
    int bytes,
    const char * comp_name,
    const char * comp_version)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE * hdl;
    limoi_component_callbacks_t callbacks;
    limoi_video_scheduler_t * scheduler;
    limoi_video_scheduler_priv_t * priv = NULL;

    LIMDBG("init component %s, version %s.\n", comp_name, comp_version);

    priv = calloc(1, sizeof(limoi_video_scheduler_priv_t));
    if (NULL == priv)
    {
        LIMERR("out of memory upon video scheduler init.\n");
        ret = OMX_ErrorInsufficientResources;
        goto bail;
    }

    ret = limoi_component_init(hComp, bytes);
    if (OMX_ErrorNone != ret) goto bail;
    hdl = (OMX_COMPONENTTYPE *)*hComp;

    scheduler = LIMOI_VIDEO_SCHEDULER(*hComp);
    scheduler->priv = priv;

    limoi_component_set_name_version(*hComp, comp_name, comp_version);
    add_port(*hComp);

    callbacks.command_prep = prep_cmd_handle;
    callbacks.command_post = post_cmd_handle;
    callbacks.main = main_handle;
    callbacks.sub = sub_handle;
    limoi_component_set_callbacks(*hComp, &callbacks);

    LIMDBG("component %s initialized.\n", comp_name);
    hdl->ComponentDeInit = limoi_video_scheduler_deinit;

 bail:
    if (OMX_ErrorNone != ret && priv) free(priv);
    return ret;
}
