/*
  Copyright (C) 2010 - 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL ffmpeg demuxer component.

  REVISION:

    1) initial creation. ----------------------------------------- 2010-01-27 MG
*/

#include <stdio.h>
#include <stdlib.h>
#include "OMX_Core.h"
#include "OMX_Types.h"
#include "OMX_Component.h"
#include "limoi-ffmpeg-demux.h"

//#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

#define COMPONENT_NAME "OMX.limoi.ffmpeg.demux"
#define ROLE_NAME "container_demuxer.all"

static ffmpeg_cb_t * ffmpeg_cb;

static int get_metadata_count (
    OMX_HANDLETYPE hComp,
    OMX_METADATASCOPETYPE scope,
    int scope_specifier,
    const char * language_country)
{
    ffmpeg_demuxer_private_t * demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    int count = 0;
    int ii;

    limoi_component_lock(hComp);

    //TODO: honor language country.
    /*
    if (language_country)
    {
        if (demuxer->metadata_language_country)
        {
            if (strcmp(language_country, demuxer->metadata_language_country))
            {
                free(demuxer->metadata_language_country);
                demuxer->metadata_language_country = strdup(language_country);
            }
        }
    }
    */

    if ((demuxer->metadata_scope != scope
        || demuxer->metadata_scope_specifier != scope_specifier)
        && demuxer->avformat)
    {
        AVDictionaryEntry * tag = NULL;
        AVStream * stream = NULL;

        switch (scope)
        {
        case OMX_MetadataScopeAllLevels :
            while ((tag = av_dict_get(demuxer->avformat->metadata,
                                      "", tag, AV_METADATA_IGNORE_SUFFIX)))
                count++;

            for (ii = 0; ii < demuxer->avformat->nb_streams; ii++)
            {
                stream = demuxer->avformat->streams[ii];
                tag = NULL;
                while ((tag = av_dict_get
                        (stream->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)))
                    count++;
            }
            break;

        case OMX_MetadataScopeTopLevel :
            while ((tag = av_dict_get(demuxer->avformat->metadata,
                                      "", tag, AV_METADATA_IGNORE_SUFFIX)))
                count++;
            break;

        case OMX_MetadataScopePortLevel :
            if (scope_specifier == 0 && demuxer->active_audio_index >= 0)
                stream = demuxer->avformat->streams[demuxer->active_audio_index];
            else if (scope_specifier == 1 && demuxer->active_video_index >= 0)
                stream = demuxer->avformat->streams[demuxer->active_video_index];

            if (stream)
            {
                tag = NULL;
                while ((tag = av_dict_get
                        (stream->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)))
                    count++;
                demuxer->metadata_count = count;
            } else count = -1;
            break;

        default :
            count = -1;
            break;
        }
    }

    if (-1 != count)
    {
        demuxer->metadata_count = count;
        demuxer->metadata_scope = scope;
        demuxer->metadata_scope_specifier = scope_specifier;
    }

    limoi_component_unlock(hComp);

    return count;
}

static OMX_ERRORTYPE get_metadata (
    OMX_HANDLETYPE hComp,
    OMX_METADATASCOPETYPE scope,
    int scope_specifier,
    const char * language_country,
    OMX_U32 index,
    OMX_U8 * key_bytes,
    void * key,
    OMX_METADATACHARSETTYPE * key_charset,
    OMX_U32 * value_bytes,
    void * value,
    OMX_METADATACHARSETTYPE * value_charset)
{
    ffmpeg_demuxer_private_t * demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    AVDictionaryEntry * tag = NULL;
    int ii;

    limoi_component_lock(hComp);

    if (demuxer->avformat)
    {
        AVStream * stream = NULL;
        int count = 0;

        switch (scope)
        {
        case OMX_MetadataScopeAllLevels :
            while ((tag = av_dict_get(demuxer->avformat->metadata,
                                      "", tag, AV_METADATA_IGNORE_SUFFIX)))
            {
                if (count == index) goto done;
                count++;
            }

            for (ii = 0; ii < demuxer->avformat->nb_streams; ii++)
            {
                stream = demuxer->avformat->streams[ii];
                tag = NULL;
                while ((tag = av_dict_get
                        (stream->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)))
                {
                    if (count == index) goto done;
                    count++;
                }
            }
            break;

        case OMX_MetadataScopeTopLevel :
            while ((tag = av_dict_get(demuxer->avformat->metadata,
                                      "", tag, AV_METADATA_IGNORE_SUFFIX)))
            {
                if (count == index) goto done;
                count++;
            }
            break;

        case OMX_MetadataScopePortLevel :
            if (scope_specifier == 0 && demuxer->active_audio_index >= 0)
                stream = demuxer->avformat->streams[demuxer->active_audio_index];
            else if (scope_specifier == 1 && demuxer->active_video_index >= 0)
                stream = demuxer->avformat->streams[demuxer->active_video_index];

            if (stream)
            {
                tag = NULL;
                while ((tag = av_dict_get
                        (stream->metadata, "", tag, AV_METADATA_IGNORE_SUFFIX)))
                {
                    if (count == index) goto done;
                    count++;
                }
            }
            break;

        default :
            break;
        }
    }

    limoi_component_unlock(hComp);
    return OMX_ErrorBadParameter;

done:
    ii = strlen(tag->key);
    *key_bytes = (ii + 1 > 128)? 128 : ii + 1;
    memcpy(key, tag->key, *key_bytes);
    *((char *)key + *key_bytes - 1) = 0;

    ii = strlen(tag->value);
    *value_bytes = (ii + 1 > *value_bytes)? *value_bytes : ii + 1;
    memcpy(value, tag->value, *value_bytes);
    *((char *)value + *value_bytes - 1) = 0;

    //TODO: charset assignment
    *key_charset = OMX_MetadataCharsetUTF8;
    *value_charset = OMX_MetadataCharsetUTF8;

    limoi_component_unlock(hComp);
    return OMX_ErrorNone;
}


static void print_error (
    int err)
{
    char errbuf[128];
    const char *errbuf_ptr = errbuf;

    if (av_strerror(err, errbuf, sizeof(errbuf)) < 0)
        errbuf_ptr = strerror(AVUNERROR(err));
    LIMERR("%s\n", errbuf_ptr);
}

static int open_active_stream (OMX_HANDLETYPE hComp)
{
    ffmpeg_demuxer_private_t * ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    OMX_PARAM_U32TYPE active_stream;
    OMX_U32 audio_index;
    OMX_U32 video_index;
    int ii;

    active_stream.nSize = sizeof(OMX_PARAM_U32TYPE);
    active_stream.nPortIndex = 0;
    if (OMX_ErrorNone !=
        OMX_GetParameter(hComp, OMX_IndexParamActiveStream,
                         &active_stream))
    {
        active_stream.nU32 = OMX_IndexMax;
    }
    audio_index = active_stream.nU32;

    active_stream.nPortIndex = 1;
    if (OMX_ErrorNone !=
        OMX_GetParameter(hComp, OMX_IndexParamActiveStream,
                         &active_stream))
    {
        active_stream.nU32 = OMX_IndexMax;
    }
    video_index = active_stream.nU32;

    for (ii = 0; ii < ffmpeg_demuxer->avformat->nb_streams; ii++)
    {
        AVCodecContext * enc = ffmpeg_demuxer->avformat->streams[ii]->codec;

        switch(enc->codec_type)
        {
        case AVMEDIA_TYPE_AUDIO:
            if (audio_index-- == 0)
            {
                ffmpeg_demuxer->active_audio_index = ii;
                LIMDBG("%s audio codec timebase %d/%d stream timebase %d/%d\n",
                       COMPONENT_NAME,
                       enc->time_base.num, enc->time_base.den,
                       ffmpeg_demuxer->avformat->streams[ii]->time_base.num,
                       ffmpeg_demuxer->avformat->streams[ii]->time_base.den);
            }
            break;

        case AVMEDIA_TYPE_VIDEO:
            if (video_index-- == 0)
            {
                ffmpeg_demuxer->active_video_index = ii;
                LIMDBG("%s video codec timebase %d/%d stream timebase %d/%d\n",
                       COMPONENT_NAME,
                       enc->time_base.num, enc->time_base.den,
                       ffmpeg_demuxer->avformat->streams[ii]->time_base.num,
                       ffmpeg_demuxer->avformat->streams[ii]->time_base.den);
            }
            break;

        case AVMEDIA_TYPE_SUBTITLE:
            //TODO:
            break;

        default:
            LIMWARN("%s unknown codec type: %d\n",
                    COMPONENT_NAME, enc->codec_type);
            break;
        }
    }
    LIMDBG("%s active audio index: %d, active video index: %d\n",
           COMPONENT_NAME,
           ffmpeg_demuxer->active_audio_index,
           ffmpeg_demuxer->active_video_index);

    return (-1 == ffmpeg_demuxer->active_audio_index
            && -1 == ffmpeg_demuxer->active_video_index)? -1 : 0;
}

static void get_content_uri (OMX_HANDLETYPE hComp)
{
    ffmpeg_demuxer_private_t * ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    void * buf[(sizeof(OMX_PARAM_CONTENTURITYPE) + LIMOI_URI_MAX_LENGTH) /
               sizeof(void *)];
    OMX_PARAM_CONTENTURITYPE * uri_param = (OMX_PARAM_CONTENTURITYPE *)&buf[0];

    limoi_set_spec_version(uri_param, sizeof(OMX_PARAM_CONTENTURITYPE)
                           + LIMOI_URI_MAX_LENGTH);
    if (OMX_ErrorNone == limoi_component_get_parameter
        (hComp, OMX_IndexParamContentURI, uri_param))
    {
        strncpy((char *)ffmpeg_demuxer->uri, (char *)uri_param->contentURI,
                LIMOI_URI_MAX_LENGTH - 1);
        ffmpeg_demuxer->uri[LIMOI_URI_MAX_LENGTH - 1] = 0;
    } else
    {
        ffmpeg_demuxer->uri[0] = 0;
    }
}

static OMX_ERRORTYPE set_state (
    limoi_component_cmd_t * cmd)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComp = cmd->hComponent;
    OMX_STATETYPE next_state = cmd->nParam1;
    OMX_STATETYPE state = limoi_component_state(hComp);
    ffmpeg_demuxer_private_t * ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(hComp);

    LIMDBG("%s callback set state from %s to %s.\n", COMPONENT_NAME,
           limoi_omx_state_string(state), limoi_omx_state_string(next_state));

    switch (state)
    {
    case OMX_StateLoaded :

        // transition to StateIdle
        ffmpeg_demuxer->bio = NULL;
        ffmpeg_demuxer->avformat= NULL;

        ffmpeg_demuxer->active_audio_index = -1;
        ffmpeg_demuxer->active_video_index = -1;
        get_content_uri(cmd->hComponent);

        ffmpeg_demuxer->metadata_scope = -1;
        ffmpeg_demuxer->metadata_scope_specifier = -1;
        ffmpeg_demuxer->metadata_count = -1;
        ffmpeg_demuxer->metadata_language_country = NULL;

        omx_ffmpeg_mux_init();
        break;

    case OMX_StateIdle :
        if (OMX_StateLoaded == next_state)
        {
            if (ffmpeg_demuxer->bio)
            {
                av_free(ffmpeg_demuxer->bio);
                ffmpeg_demuxer->bio = NULL;
            }
            if (ffmpeg_demuxer->avformat)
            {
                av_close_input_stream(ffmpeg_demuxer->avformat);
                ffmpeg_demuxer->avformat = NULL;
            }

            if (ffmpeg_demuxer->metadata_language_country)
            {
                free(ffmpeg_demuxer->metadata_language_country);
                ffmpeg_demuxer->metadata_language_country = NULL;
            }
        }
        break;

    default :
        break;
    }

    return ret;
}

static OMX_ERRORTYPE cmd_handle (
    limoi_component_cmd_t * cmd)
{
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_HANDLETYPE hComp = cmd->hComponent;

    LIMDBG("%s command handler.\n", COMPONENT_NAME);
    limoi_component_lock(hComp);
    switch (cmd->Cmd)
    {
    case OMX_CommandStateSet :
        ret = set_state(cmd);
        break;

    default :
        break;
    }
    limoi_component_unlock(hComp);
    return ret;
}

static CPresult content_pipe_cb(
    CP_EVENTTYPE eEvent,
    CPuint iParam)
{
    switch (eEvent)
    {
    case CP_BytesAvailable :
        limoi_component_lock(ffmpeg_cb->hComp);
        ffmpeg_cb->bytes_available = iParam;
        limoi_component_cond_broadcast(ffmpeg_cb->hComp);
        limoi_component_unlock(ffmpeg_cb->hComp);
        break;

    default :
        LIMWARN("%s unhanlded content pipe callback.\n", COMPONENT_NAME);
        break;
    }

    return 0;
}

static int read_packet_cb (
    void * opaque,
    uint8_t * buf,
    int buf_size)
{
    ffmpeg_cb_t * cb = (ffmpeg_cb_t *)opaque;

    //LIMDBG("%s read bytes: %d\n", COMPONENT_NAME, buf_size);
    while (cb->pipe->Read(cb->content, (CPbyte *)buf, (CPuint)buf_size))
    {
        CP_CHECKBYTESRESULTTYPE res;
    retry:
        cb->bytes_available = 0;
        cb->pipe->CheckAvailableBytes(cb->content, buf_size, &res);

        if (CP_CheckBytesOk == res)
        {
            continue;
        } else if (CP_CheckBytesNotReady == res)
        {
            while (!cb->bytes_available)
            {
                limoi_component_cond_timedwait(cb->hComp, -1);
                //we may now reach EOS
                if (!cb->bytes_available) goto retry;
            }
        } else if (CP_CheckBytesInsufficientBytes == res)
        {
            while (!cb->bytes_available)
            {
                limoi_component_cond_timedwait(cb->hComp, -1);
            }
            buf_size = (cb->bytes_available > buf_size)?
                buf_size : cb->bytes_available;
        } else //EOF
        {
            return 0;
        }
    }

    return buf_size;
}

static int64_t seek_cb (
    void * opaque,
    int64_t offset,
    int whence)
{
    CP_ORIGINTYPE orig;
    ffmpeg_cb_t * cb = (ffmpeg_cb_t *)opaque;

    LIMDBG("%s seeking to: %"PRId64" whence: %d\n", COMPONENT_NAME,
           offset, whence);
    if (SEEK_CUR == whence)
        orig = CP_OriginCur;
    else if (SEEK_SET == whence)
        orig = CP_OriginBegin;
    else if (SEEK_END == whence)
        orig = CP_OriginEnd;
    else //if(AVSEEK_SIZE == whence): feature not supported.
        return -1;

    if (!cb->pipe->SetPosition(cb->content, (CPint)offset, orig))
    {
        CPuint ret;

        cb->pipe->GetPosition(cb->content, &ret);
        LIMDBG("%s seek return as: %"PRId64"\n", COMPONENT_NAME, ret);
        return ret;
    }

    return -1;
}

/** size of probe buffer, for guessing file type from file contents */
#define PROBE_BUF_MIN 2048
#define PROBE_BUF_MAX (1 << 20)
static AVInputFormat * probe_format (ffmpeg_demuxer_private_t * ffmpeg_demuxer)
{
    AVIOContext * bio = ffmpeg_demuxer->bio;
    AVInputFormat * fmt = NULL;
    int probe_size;
    AVProbeData probe_data, * pd = &probe_data;

    if (!fmt)
    {
        //INVESTIGATION AND FIXME:
        //if fmt is detected through this path, the parent
        //class supplied content-pipe will NOT be used, thus
        //may suffer performance penalty.
        pd->buf = NULL;
        pd->filename = (char *)ffmpeg_demuxer->uri;
        pd->buf_size = 0;
        fmt = av_probe_input_format(pd, 0);
        if (fmt) LIMDBG("%s format probe done.\n", COMPONENT_NAME);
    }

    if (!fmt)
    {
        LIMDBG("%s further format probe.\n", COMPONENT_NAME);
        pd->buf = NULL;
        pd->filename = "";
        for (probe_size = PROBE_BUF_MIN;
             probe_size <= PROBE_BUF_MAX && !fmt;
             probe_size <<= 1)
        {
            /* read probe data */
            pd->buf= av_realloc(pd->buf, probe_size + AVPROBE_PADDING_SIZE);
            pd->buf_size = avio_read(bio, pd->buf, probe_size);

            if ((int)pd->buf_size < 0) break;

            memset(pd->buf + pd->buf_size, 0, AVPROBE_PADDING_SIZE);
            avio_seek(bio, 0, SEEK_SET);

            /* guess file format */
            fmt = av_probe_input_format(pd, 1);
            if(fmt) break;
        }
        av_freep(&pd->buf);
    }

    return fmt;
}

static OMX_ERRORTYPE clock_handle (
    OMX_HANDLETYPE hComp,
    OMX_HANDLETYPE clock_tunnel_comp,
    OMX_BUFFERHEADERTYPE * clock_buf)
{
    LIMDBG("%s: clock handle.\n", COMPONENT_NAME);
    clock_buf->nFilledLen = 0;

    return OMX_ErrorNone;
}

static OMX_ERRORTYPE demux_handle (
    OMX_HANDLETYPE hComp,
    CP_PIPETYPE * pipe,
    CPhandle content,
    OMX_BUFFERHEADERTYPE * audio_out,
    OMX_BUFFERHEADERTYPE * video_out)
{
    ffmpeg_demuxer_private_t * ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    OMX_ERRORTYPE res = OMX_ErrorNone;
    int num_audio_streams = 0;
    int num_video_streams = 0;
#define MAX_AUDIO_STREAM_NUM__ 16
#define MAX_VIDEO_STREAM_NUM__ 8
    limoi_audio_format_t audio_format[MAX_AUDIO_STREAM_NUM__];
    limoi_video_format_t video_format[MAX_AUDIO_STREAM_NUM__];

    limoi_component_lock(hComp);
    ffmpeg_cb = &ffmpeg_demuxer->cb;

    if (ffmpeg_demuxer->bio == NULL)
    {
        AVInputFormat * iformat;
        int err;
        int ii;

        pipe->RegisterCallback(content, content_pipe_cb);

        ffmpeg_cb->pipe = pipe;
        ffmpeg_cb->content = content;
        ffmpeg_cb->hComp = hComp;

        ffmpeg_demuxer->bio = avio_alloc_context
            (ffmpeg_demuxer->bio_buffer,
             FFMPEG_BIO_BUFFER_SIZE,
             0,
             (void *)ffmpeg_cb,
             read_packet_cb,
             NULL,
             seek_cb);

        iformat = probe_format(ffmpeg_demuxer);
        if (NULL == iformat)
        {
            LIMERR("%s unable to detect input format.\n", COMPONENT_NAME);
            res = OMX_ErrorFormatNotDetected;
            goto bail;
        } else
        {
            LIMDBG("%s AV input format:\n\t%s\n\t%s\n", COMPONENT_NAME,
                   iformat->name, iformat->long_name);

            LIMDBG("av format structure\n"
                   "\tflags          : %x\n"
                   "\tpriv_data_size : %d\n"
                   "\textensions     : %s\n"
                   "\tvalue          : %d\n"
                   "\tcodec tag      : %p\n"
                   "\tmetadata_conv  : %p\n"
                   "\tnext           : %p\n",
                   iformat->flags, iformat->priv_data_size,
                   iformat->extensions, iformat->value,
                   iformat->codec_tag, iformat->metadata_conv,
                   iformat->next);
        }

        ffmpeg_demuxer->avformat = avformat_alloc_context();
        ffmpeg_demuxer->avformat->pb = ffmpeg_demuxer->bio;

        err = avformat_open_input(&ffmpeg_demuxer->avformat,
                                  (char *)ffmpeg_demuxer->uri,
                                  iformat, NULL);
        if (err < 0)
        {
            LIMERR("%s unable to open stream.\n", COMPONENT_NAME);
            print_error(err);
            res = OMX_ErrorFormatNotDetected;
            goto bail;
        }

        err = av_find_stream_info(ffmpeg_demuxer->avformat);
        if (err < 0)
        {
            LIMERR("%s unable to find stream info.\n", COMPONENT_NAME);
            print_error(err);
            res = OMX_ErrorFormatNotDetected;
            goto bail;
        }

        if (ffmpeg_demuxer->avformat->nb_streams < 1)
        {
            LIMERR("%s no streams found.\n", COMPONENT_NAME);
            res = OMX_ErrorFormatNotDetected;
            goto bail;
        }

        //dump_format(ffmpeg_demuxer->avformat, 0, "", 0);

        for (ii = 0; ii < ffmpeg_demuxer->avformat->nb_streams; ii++)
        {
            AVCodecContext * enc = ffmpeg_demuxer->avformat->streams[ii]->codec;

            switch(enc->codec_type)
            {
            case AVMEDIA_TYPE_AUDIO:
                num_audio_streams++;
                ASSERT(num_audio_streams < MAX_AUDIO_STREAM_NUM__);
                audio_format[num_audio_streams - 1].coding_type =
                    omx_ffmpeg_codec_id_ffmpeg2omx(enc->codec_id);
                break;

            case AVMEDIA_TYPE_VIDEO:
                num_video_streams++;
                ASSERT(num_video_streams < MAX_VIDEO_STREAM_NUM__);
                video_format[num_video_streams - 1].coding_type =
                    omx_ffmpeg_codec_id_ffmpeg2omx(enc->codec_id);
                video_format[num_video_streams - 1].color_format =
                    OMX_COLOR_FormatUnused;
                if (enc->time_base.num)
                {
                    video_format[num_video_streams - 1].framerate =
                        enc->time_base.den / enc->time_base.num;
                } else
                {
                    LIMWARN("%s unable to determine stream %d framerate.\n",
                            COMPONENT_NAME, ii);
                }
                break;

            case AVMEDIA_TYPE_SUBTITLE:
                //TODO:
                break;

            default:
                LIMWARN("%s unknown codec type: %d\n",
                        COMPONENT_NAME, enc->codec_type);
                break;
            }
        }
        limoi_demuxer_set_port_params(hComp, num_audio_streams, audio_format,
                                      num_video_streams, video_format);

    } else if (-1 == ffmpeg_demuxer->active_audio_index
               && -1 == ffmpeg_demuxer->active_video_index)
    {
        open_active_stream(hComp);
    } else if (audio_out || video_out)
    {
        int retry_count = 3;
        int ret;
        AVPacket pkt;
        limoi_port_t * limoi_port;

    retry:
        ret = av_read_frame(ffmpeg_demuxer->avformat, &pkt);

        if (ret < 0)
        {
            if (ret != AVERROR_EOF && retry_count--)
            {
                //LIMWARN("%s unknown error %d with pkt index: %d size: %d.\n",
                //        COMPONENT_NAME, ret, pkt.stream_index, pkt.size);
                goto retry;
            } else
            {
                LIMDBG("%s EOS reached with pkt index: %d size: %d.\n",
                       COMPONENT_NAME, pkt.stream_index, pkt.size);
                audio_out->nFilledLen = 0;
                audio_out->nFlags |= OMX_BUFFERFLAG_EOS;
                video_out->nFilledLen = 0;
                video_out->nFlags |= OMX_BUFFERFLAG_EOS;
                goto bail;
            }
        }

        if (ret >= 0)
        {
            if (pkt.stream_index == ffmpeg_demuxer->active_audio_index)
            {
                LIMDBG("%s audio nAllocLen = %u, pkt.size = %d\n",
                       COMPONENT_NAME, audio_out->nAllocLen, pkt.size);
                if (audio_out->nAllocLen < pkt.size)
                {
                    limoi_port = limoi_component_get_port
                        (hComp, audio_out->nOutputPortIndex);

                    if (NULL == limoi_port)
                    {
                        LIMERR("%s unable to get audio port.\n",
                               COMPONENT_NAME);
                        res = OMX_ErrorUndefined;
                        goto bail_free_packet;
                    } else
                    {
                        if (limoi_component_realloc_port_buffer
                            (hComp, limoi_port, audio_out, pkt.size))
                        {
                            LIMERR("%s unable to realloc audio port buffer.\n",
                                   COMPONENT_NAME);
                            res = OMX_ErrorUndefined;
                            goto bail_free_packet;
                        }
                    }
                }
                memcpy(audio_out->pBuffer, pkt.data, pkt.size);
                audio_out->nFilledLen = pkt.size;
                audio_out->nTimeStamp = pkt.pts;
                LIMDBG("audio pts at demuxer: %"PRId64"\n", pkt.pts);

                //associate the stream context with port.
                audio_out->pOutputPortPrivate =
                    ffmpeg_demuxer->avformat->streams
                    [ffmpeg_demuxer->active_audio_index];
            } else if (pkt.stream_index == ffmpeg_demuxer->active_video_index)
            {
                LIMDBG("%s video nAllocLen = %u, pkt.size = %d\n",
                       COMPONENT_NAME, video_out->nAllocLen, pkt.size);
                if (video_out->nAllocLen < pkt.size + sizeof(AVPacket))
                {
                    limoi_port = limoi_component_get_port
                        (hComp, video_out->nOutputPortIndex);

                    if (NULL == limoi_port)
                    {
                        LIMERR("%s unable to get video port.\n",
                               COMPONENT_NAME);
                        res = OMX_ErrorUndefined;
                        goto bail_free_packet;
                    } else
                    {
                        if (limoi_component_realloc_port_buffer
                            (hComp, limoi_port,
                             video_out, pkt.size + sizeof(AVPacket)))
                        {
                            LIMERR("%s unable to realloc video port buffer.\n",
                                   COMPONENT_NAME);
                            res = OMX_ErrorUndefined;
                            goto bail_free_packet;
                        }
                    }
                }
                memcpy(video_out->pBuffer, pkt.data, pkt.size);
                video_out->nFilledLen = pkt.size;
                //video_out->nTimeStamp = pkt.pts;
                LIMDBG("video pts at demuxer: %"PRId64"\n", pkt.pts);
                memcpy(video_out->pBuffer + pkt.size, &pkt, sizeof(AVPacket));

                //associate the stream context with port.
                video_out->pOutputPortPrivate =
                    ffmpeg_demuxer->avformat->streams
                    [ffmpeg_demuxer->active_video_index];
            }
        bail_free_packet:
            av_free_packet(&pkt);
        }
    }

 bail:
    limoi_component_unlock(hComp);
    return res;
}

static OMX_ERRORTYPE SetConfig (
    OMX_HANDLETYPE hComp,
    OMX_INDEXTYPE nIndex,
    OMX_PTR pCompConfigStruct)
{
    ffmpeg_demuxer_private_t * ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(hComp);
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    limoi_component_lock(hComp);
    switch (nIndex)
    {
    case OMX_IndexConfigTimePosition :
        {
            OMX_TIME_CONFIG_TIMESTAMPTYPE * ts;
            int64_t target;

            ts = (OMX_TIME_CONFIG_TIMESTAMPTYPE *)pCompConfigStruct;
            target = ((ts->nTimestamp / 1000) * AV_TIME_BASE) / 1000;
            LIMDBG("%s seeking to %"PRId64" %"PRId64" %"PRId64" %"PRId64"\n",
                   COMPONENT_NAME, ts->nTimestamp, target,
                   INT64_MIN, INT64_MAX);

            //TODO: check and honore seek mode here.
            //TODO: compare existing timestamp and assign smalled seek range
            if (0 > avformat_seek_file
                (ffmpeg_demuxer->avformat, -1,
                 INT64_MIN, target, INT64_MAX, 0))
            {
                int64_t byte_target;

                LIMINFO("%s fall back to byte seeking.\n", COMPONENT_NAME);

                //fallback to byte seek.
                byte_target = (target - ffmpeg_demuxer->avformat->start_time)
                    * ffmpeg_demuxer->avformat->bit_rate / (8 * AV_TIME_BASE);
                LIMDBG("%s fallback seek by bytes\n"
                       "\ttarget     : %"PRId64
                       "\tstart time : %"PRId64
                       "\tbitrate    : %d\n"
                       "\tbyte target : %"PRId64"\n",
                       COMPONENT_NAME, target,
                       ffmpeg_demuxer->avformat->start_time,
                       ffmpeg_demuxer->avformat->bit_rate,
                       byte_target);

                if (0 > avformat_seek_file
                    (ffmpeg_demuxer->avformat,
                     -1, INT64_MIN, byte_target, INT64_MAX,
                     AVSEEK_FLAG_BYTE))
                {
                    LIMERR("unable to seek in demuxer.\n");
                    ret = OMX_ErrorUndefined;
                }
            }
        }
        break;

    default :
        ret = limoi_component_set_parameter(hComp, nIndex, pCompConfigStruct);
        break;
    }

    limoi_component_unlock(hComp);
    return ret;
}

static OMX_ERRORTYPE ComponentDeInit (
    OMX_HANDLETYPE hComp)
{
    return limoi_demuxer_deinit(hComp);
}

static OMX_ERRORTYPE ComponentInit (OMX_HANDLETYPE * hComp, const char * name)
{
    OMX_COMPONENTTYPE * hdl;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    limoi_demuxer_callbacks_t callbacks;
    ffmpeg_demuxer_private_t * ffmpeg_demuxer;

    LIMDBG("loading %s component.\n", COMPONENT_NAME);
    ret = limoi_demuxer_init(hComp, sizeof(ffmpeg_demuxer_private_t),
                             COMPONENT_NAME, VERSION);
    if (OMX_ErrorNone != ret) goto bail;
    hdl = (OMX_COMPONENTTYPE *)*hComp;

    ffmpeg_demuxer = LIMOI_FFMPEG_DEMUXER(*hComp);

    limoi_component_add_role(*hComp, ROLE_NAME);

    hdl->ComponentDeInit = ComponentDeInit;
    hdl->SetConfig = SetConfig;

    callbacks.command_prep = NULL;
    callbacks.command_post = cmd_handle;
    callbacks.demux = demux_handle;
    callbacks.clock = clock_handle;
    callbacks.get_metadata_count = get_metadata_count;
    callbacks.get_metadata = get_metadata;
    limoi_demuxer_set_callbacks(*hComp, &callbacks);

    LIMDBG("%s component loaded.\n", COMPONENT_NAME);

 bail:
    return ret;
}

lim_omx_component_registration_t ** OMX_ComponentRegistration (int * num)
{
    static char * roles[] = {ROLE_NAME, NULL};
    static lim_omx_component_registration_t comp_reg = {
        COMPONENT_NAME, roles, ComponentInit
    };
    static lim_omx_component_registration_t * regs[] = {&comp_reg};

    *num = 1;
    return regs;
}
