/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX ffmpeg codec wrapper.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-03-10 MG
*/

#include "omx-ffmpeg.h"

typedef struct
{
    int ffmpeg_codec_id;
    int omx_codec_id;
} ffmpeg_omx_codec_id_mapping_t;

static ffmpeg_omx_codec_id_mapping_t FFMPEG_OMX_AUDIO_CODEC_ID_MAPPING_TAB[] = {
    {CODEC_ID_PCM_S16LE, OMX_AUDIO_CodingPCM},
    {CODEC_ID_PCM_S16BE, OMX_AUDIO_CodingPCM},
    {CODEC_ID_ADPCM_IMA_WAV, OMX_AUDIO_CodingADPCM},
    {CODEC_ID_AMR_NB, OMX_AUDIO_CodingAMR},
    {CODEC_ID_AMR_WB, OMX_AUDIO_CodingAMR},
    {CODEC_ID_MP2, OMX_AUDIO_CodingMP3},
    {CODEC_ID_MP3, OMX_AUDIO_CodingMP3},
    {CODEC_ID_AAC, OMX_AUDIO_CodingAAC},
    {CODEC_ID_VORBIS, OMX_AUDIO_CodingVORBIS},
    {CODEC_ID_WMAV1, OMX_AUDIO_CodingWMA},
    {CODEC_ID_WMAV2, OMX_AUDIO_CodingWMA},
    {CODEC_ID_RA_144, OMX_AUDIO_CodingRA},
    {CODEC_ID_RA_288, OMX_AUDIO_CodingRA},
    {CODEC_ID_COOK, OMX_AUDIO_CodingRA},

    {CODEC_ID_NONE, OMX_AUDIO_CodingMax}
};

static ffmpeg_omx_codec_id_mapping_t FFMPEG_OMX_VIDEO_CODEC_ID_MAPPING_TAB[] = {
    {CODEC_ID_MPEG1VIDEO, OMX_VIDEO_CodingMPEG2},
    {CODEC_ID_MPEG2VIDEO, OMX_VIDEO_CodingMPEG2},
    {CODEC_ID_H263, OMX_VIDEO_CodingH263},
    {CODEC_ID_MPEG4, OMX_VIDEO_CodingMPEG4},
    {CODEC_ID_WMV1, OMX_VIDEO_CodingWMV},
    {CODEC_ID_WMV2, OMX_VIDEO_CodingWMV},
    {CODEC_ID_H264, OMX_VIDEO_CodingAVC},
    {CODEC_ID_MJPEG, OMX_VIDEO_CodingMJPEG},
    {CODEC_ID_RV10, OMX_VIDEO_CodingRV},
    {CODEC_ID_RV20, OMX_VIDEO_CodingRV},

    {CODEC_ID_NONE, OMX_VIDEO_CodingMax}
};

int omx_ffmpeg_audio_codec_id_omx2ffmpeg (int omx_codec_id)
{
    ffmpeg_omx_codec_id_mapping_t * map;

    map = &FFMPEG_OMX_AUDIO_CODEC_ID_MAPPING_TAB[0];
    while (map->omx_codec_id != OMX_AUDIO_CodingMax)
    {
        if (map->omx_codec_id == omx_codec_id) return map->ffmpeg_codec_id;
        else map++;
    }

    LIMERR("unmapped codec id: %d\n", omx_codec_id);
    return CODEC_ID_NONE;
}

int omx_ffmpeg_video_codec_id_omx2ffmpeg (int omx_codec_id)
{
    ffmpeg_omx_codec_id_mapping_t * map;

    map = &FFMPEG_OMX_VIDEO_CODEC_ID_MAPPING_TAB[0];
    while (map->omx_codec_id != OMX_VIDEO_CodingMax)
    {
        if (map->omx_codec_id == omx_codec_id) return map->ffmpeg_codec_id;
        else map++;
    }

    LIMERR("unmapped codec id: %d\n", omx_codec_id);
    return CODEC_ID_NONE;
}

int omx_ffmpeg_codec_id_ffmpeg2omx (int ffmpeg_codec_id)
{
    ffmpeg_omx_codec_id_mapping_t * map;

    map = &FFMPEG_OMX_AUDIO_CODEC_ID_MAPPING_TAB[0];
    while (map->ffmpeg_codec_id != CODEC_ID_NONE)
    {
        if (map->ffmpeg_codec_id == ffmpeg_codec_id) return map->omx_codec_id;
        else map++;
    }

    map = &FFMPEG_OMX_VIDEO_CODEC_ID_MAPPING_TAB[0];
    while (map->ffmpeg_codec_id != CODEC_ID_NONE)
    {
        if (map->ffmpeg_codec_id == ffmpeg_codec_id) return map->omx_codec_id;
        else map++;
    }

    if (ffmpeg_codec_id > CODEC_ID_NONE
        && ffmpeg_codec_id < CODEC_ID_PCM_S16LE)
    {
        //video codec catch all.
        LIMWARN("unable to map id: %d, default to OMX_VIDEO_CodingMPEG2\n",
                ffmpeg_codec_id);
        return OMX_VIDEO_CodingMPEG2;
    } else if (ffmpeg_codec_id >= CODEC_ID_PCM_S16LE
               && ffmpeg_codec_id < CODEC_ID_DVD_SUBTITLE)
    {
        //audio codec catch all.
        LIMWARN("unable to map id: %d, default to OMX_AUDIO_CodingMP3\n",
                ffmpeg_codec_id);
        return OMX_AUDIO_CodingMP3;
    }

    LIMERR("unmapped codec id: %d\n", ffmpeg_codec_id);
    return OMX_AUDIO_CodingMax;
}

//ffmpeg avcodec lock manager callback.
static lthread_mutex_t avcodec_mutex = LTHREAD_MUTEX_INITIALIZER;
static int avcodec_inited = 0;
static int av_lock_cb (void ** mutex, enum AVLockOp op)
{
    switch (op)
    {
    case AV_LOCK_CREATE :
        *mutex = &avcodec_mutex;
        break;

    case AV_LOCK_OBTAIN :
        lthread_mutex_lock(&avcodec_mutex);
        break;

    case AV_LOCK_RELEASE :
        lthread_mutex_unlock(&avcodec_mutex);
        break;

    case AV_LOCK_DESTROY :
        break;
    }
    return 0;
}

int omx_ffmpeg_codec_init (void)
{
    lthread_mutex_lock(&avcodec_mutex);
    if (avcodec_inited)
    {
        lthread_mutex_unlock(&avcodec_mutex);
        return 0;
    } else avcodec_inited = 1;
    lthread_mutex_unlock(&avcodec_mutex);

    avcodec_init();
    avcodec_register_all();

    av_lockmgr_register(av_lock_cb);
    LIMDBG("ffmpeg codec inited.\n");

    return 0;
}
