/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL LIM resource manager init/deinit module.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-04-29 MG
*/
#define _GNU_SOURCE
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>
#include "resource-manager.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

typedef struct
{
    list_t list;
    void * lib; /** non-null library handle if loaded. */
} rm_reg_t;

static list_t rm_list_head;

static void load_rm (const char * cname, void * arg)
{
	void * so = NULL;
	void * sym;
    rm_reg_t * reg = NULL;

    LIMDBG("cname: %s\n", cname);
	so = dlopen(cname, RTLD_LAZY);

    if (so)
    {
        sym = dlsym(so, "LIM_ResourceManagerDeinit");
        if (NULL == sym) goto bail;

        sym = dlsym(so, "LIM_ResourceManagerInit");
        if (NULL == sym) goto bail;

        //typedef void (*LIM_ResourceManagerInit) (void);
        ((void (*)(void))sym)();

        reg = malloc(sizeof(rm_reg_t));
        if (NULL == reg)
        {
            LIMERR("out of memory, unable to register resource manager.\n");
            goto bail;
        }

        reg->lib = so;
        list_init(&reg->list);
        list_add(&reg->list, &rm_list_head);

        *(int *)arg = *(int *)arg + 1;
        return;
    }

 bail:
    if (reg) free(reg);
	if (so) dlclose(so);
}

void lim_resource_manager_init (void)
{
    int num_rms = 0;

    list_init(&rm_list_head);

    limdir_recursive_scan(resourcemanagersdir, load_rm, &num_rms);

    LIMDBG("number of resource managers found: %d\n", num_rms);
}

void lim_resource_manager_deinit (void)
{
    list_t * list = rm_list_head.next;
    rm_reg_t * reg;
    void * sym;

    while (list != &rm_list_head)
    {
        reg = (rm_reg_t *)list;

        ASSERT(reg->lib);
        sym = dlsym(reg->lib, "LIM_ResourceManagerDeinit");
        ASSERT(sym);

        //typedef void (*LIM_ResourceManagerDeinit) (void);
        ((void (*)(void))sym)();

        dlclose(reg->lib);

        list = list->next;
        list_del(&reg->list);

        free(reg);
    }
}
