/*
  Copyright (C) 2009 - 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL core component role module.

  REVISION:

    2) static and dynamic loader support. ------------------------ 2011-05-25 MG
    1) initial creation. ----------------------------------------- 2009-11-22 MG
*/

#include "core.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static int component_has_role (component_reg_t * comp_reg,
                               const char * role_name)
{
    list_t * rlist = comp_reg->roles_list_head.next;

    while (rlist != &comp_reg->roles_list_head)
    {
        int res = strncmp(role_name, ((role_reg_t *)rlist)->name,
                          OMX_MAX_STRINGNAME_SIZE);
        if (!res) return 1;
        else if (res < 0) return 0;
        rlist = rlist->next;
    }

    return 0;
}

OMX_ERRORTYPE OMX_GetComponentsOfRole (
    OMX_STRING role,
    OMX_U32 *pNumComps,
    OMX_U8  **compNames)
{
    list_t * list = limoi_omx.components_list_head.next;
    int num_comps = 0;
    OMX_ERRORTYPE ret = OMX_ErrorNone;

    while (list != &limoi_omx.components_list_head)
    {
        if (component_has_role((component_reg_t *)list, (const char *)role))
        {
            num_comps++;
        }
        list = list->next;
    }
    if (num_comps == 0)
    {
        LIMWARN("no components found for role: %s\n", role);
        return OMX_ErrorUndefined;
    }

    if (NULL == compNames)
    {
        *pNumComps = num_comps;
        ret = (num_comps <= 0)? OMX_ErrorUndefined : OMX_ErrorNone;
        goto bail;
    }

    *pNumComps = (*pNumComps > num_comps)? num_comps : *pNumComps;
    if (*pNumComps <= 0)
    {
        //ret = OMX_ErrorBadParameter;
        //goto bail;
        LIMWARN("*pNumComps has invalid input value: %u, defaults to 1.\n",
                *pNumComps);
        *pNumComps = 1;
    }
    num_comps = *pNumComps;

    list = limoi_omx.components_list_head.next;
    while (list != &limoi_omx.components_list_head)
    {
        if (component_has_role((component_reg_t *)list, (const char *)role))
        {
            strcpy((char *)*compNames, ((component_reg_t *)list)->name);
            LIMDBG("component name: %s\n", (char *)*compNames);

            if (--num_comps == 0) break;
            compNames++;
        }
        list = list->next;
    }

 bail:
    return ret;
}

OMX_ERRORTYPE OMX_GetRolesOfComponent (
	OMX_STRING compName,
    OMX_U32 *pNumRoles,
    OMX_U8 **roles)
{
    component_reg_t * reg;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    list_t * list;
    int num_roles = 0;

    reg = get_comp_registration_by_name(compName);
    if (NULL == reg)
    {
        LIMDBG("component: %s not found.\n", compName);
        ret = OMX_ErrorComponentNotFound;
        goto bail;
    }

    list = reg->roles_list_head.next;
    while (list != &reg->roles_list_head)
    {
        num_roles++;
        list = list->next;
    }

    if (NULL == roles)
    {
        *pNumRoles = num_roles;
        ret = (num_roles <= 0)? OMX_ErrorComponentNotFound : OMX_ErrorNone;
        goto bail;
    }

    *pNumRoles = (*pNumRoles > num_roles)? num_roles : *pNumRoles;
    if (*pNumRoles == 0)
    {
        ret = OMX_ErrorBadParameter;
        goto bail;
    }
    num_roles = *pNumRoles;

    list = reg->roles_list_head.next;
    while (list != &reg->roles_list_head)
    {
        strcpy((char *)*roles, ((role_reg_t *)list)->name);
        LIMDBG("role name: %s\n", (char *)*roles);

        if (--num_roles == 0) break;

        roles++;
        list = list->next;
    }

 bail:
    return ret;
}
