/*
  Copyright (C) 2009 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL core tunnel setup module.

  REVISION:

    1) initial creation. ----------------------------------------- 2009-11-22 MG
*/

#include "core.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

OMX_ERRORTYPE OMX_SetupTunnel (
    OMX_HANDLETYPE hOutput,
    OMX_U32 nPortOutput,
    OMX_HANDLETYPE hInput,
    OMX_U32 nPortInput)
{
    OMX_TUNNELSETUPTYPE tunnel_setup;
    OMX_ERRORTYPE ret = OMX_ErrorNone;
    OMX_COMPONENTTYPE * comp_in;
    OMX_COMPONENTTYPE * comp_out;

    if (NULL == hOutput || NULL == hInput)
    {
        //TODO: changing port to non-tunneled operation.
        LIMWARN("feature is not implemented.\n");
        return OMX_ErrorNotImplemented;
    }

    comp_in = (OMX_COMPONENTTYPE *)hInput;
    comp_out = (OMX_COMPONENTTYPE *)hOutput;

    tunnel_setup.eSupplier = OMX_BufferSupplyMax;
    ret = comp_out->ComponentTunnelRequest(hOutput, nPortOutput,
                                           hInput, nPortInput,
                                           &tunnel_setup);

    if (OMX_ErrorNone == ret)
    {
        ret = comp_in->ComponentTunnelRequest(hInput, nPortInput,
                                              hOutput, nPortOutput,
                                              &tunnel_setup);
        if (OMX_ErrorNone != ret)
        {
            LIMERR("unable to setup tunnel on input component.\n");
            comp_out->ComponentTunnelRequest(hOutput, nPortOutput,
                                             NULL, 0, NULL);
        }
    } else
    {
        LIMERR("unable to setup tunnel on output component.\n");
    }

    return ret;
}
