/*
  Copyright (C) 2010 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL core test code.

  REVISION:

    1) initial creation. ----------------------------------------- 2010-02-18 MG
*/

#include "stdio.h"
#include "stdlib.h"
#include "OMX_Core.h"
#include "OMX_Types.h"
#include "OMX_ContentPipe.h"
#include "OMX_Component.h"
#include "limlog.h"

static void print_component_roles (const char * comp)
{
    OMX_U32 num_roles;
    int ii;
    OMX_U8 ** names = NULL;

    if (OMX_ErrorNone !=
        OMX_GetRolesOfComponent((OMX_STRING)comp, &num_roles, NULL))
    {
        LIMERR("no roles available for component: %s\n", comp);
        goto bail;
    }

    names = (OMX_U8 **)malloc(num_roles * sizeof(OMX_U8 *));
    for (ii = 0; ii < num_roles; ii++)
    {
        names[ii] = (OMX_U8 *)malloc(sizeof(OMX_U8) * OMX_MAX_STRINGNAME_SIZE);
    }

    if (OMX_ErrorNone !=
        OMX_GetRolesOfComponent((OMX_STRING)comp, &num_roles, names))
    {
        LIMERR("can not retrieve roles for component: %s\n", comp);
        goto bail;
    }

    printf("\troles for component: %s\n", comp);
    for (ii = 0; ii < num_roles; ii++)
    {
        printf("\t[%02d]: %s\n", ii + 1, names[ii]);
    }

 bail:
    if (names)
    {
        for (ii = 0; ii < num_roles; ii++)
        {
            free(names[ii]);
        }
        free(names);
    }
}

static void print_components_by_name (void)
{
    int ii;
    OMX_ERRORTYPE ret;
    char name[OMX_MAX_STRINGNAME_SIZE];

    // print a list of IL components.
    ret = OMX_ErrorNone;

    for (ii = 0; OMX_ErrorNone == ret; ii++)
    {
        ret = OMX_ComponentNameEnum(name, OMX_MAX_STRINGNAME_SIZE, ii);
        if (OMX_ErrorNone == ret)
        {
            printf("OMX-IL component: %02d : %s\n", ii, name);
            print_component_roles(name);
        }
    }
}

static void print_role_components (const char * role)
{
    OMX_U32 num_comps;
    int ii;
    OMX_U8 ** names = NULL;

    if (OMX_ErrorNone !=
        OMX_GetComponentsOfRole((OMX_STRING)role, &num_comps, NULL))
    {
        LIMERR("no component available for role: %s\n", role);
        goto bail;
    }

    names = (OMX_U8 **)malloc(num_comps * sizeof(OMX_U8 *));
    for (ii = 0; ii < num_comps; ii++)
    {
        names[ii] = (OMX_U8 *)malloc(sizeof(OMX_U8) * OMX_MAX_STRINGNAME_SIZE);
    }

    if (OMX_ErrorNone !=
        OMX_GetComponentsOfRole((OMX_STRING)role, &num_comps, names))
    {
        LIMERR("can not retrieve components for role: %s\n", role);
        goto bail;
    }

    printf("components for roles: %s\n", role);
    for (ii = 0; ii < num_comps; ii++)
    {
        printf("\t[%02d]: %s\n", ii + 1, names[ii]);
    }

 bail:
    if (names)
    {
        for (ii = 0; ii < num_comps; ii++)
        {
            free(names[ii]);
        }
        free(names);
    }
}

static void print_components_by_role (void)
{
    //print_role_components("image_decoder.jpeg");
    print_role_components("audio_decoder.ogg");
    print_role_components("audio_renderer.pcm");
    //print_role_components("clock.binary");
}

static void my_log_print (const void * arg, int level,
                          const void * log, int bytes)
{
    printf("%s", (char *)log);
}

int main (int argc, char ** argv)
{
    lim_log_set_level(LIM_LOG_DEBUG);
    //lim_log_set_level(LIM_LOG_INFO);
    lim_log_set_listener(my_log_print, NULL);

    OMX_Init();

    print_components_by_name();
    print_components_by_role();

    OMX_Deinit();
    return 0;
}
