/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  OpenMAX IL demo resource manager.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-05-01 MG
*/
#define _GNU_SOURCE
#include "lim-demo-resource-manager.h"

#define DEMO_NUM_RESOURCES 2

typedef struct
{
    int available;
    //actual resources go here
} arbitrary_resource_t1;

typedef struct
{
    int free;
    //actual resoureces go here
} arbitrary_resource_t2;

static lthread_mutex_t mutex = LTHREAD_MUTEX_INITIALIZER;
static arbitrary_resource_t1 resource_1;
static arbitrary_resource_t2 resource_2;

static void * get_resource (int manager_id, int resource_id, void * arg)
{
    void * res = NULL;

    ASSERT(DEMO_RESOURCE_MANAGER_ID == manager_id);
    ASSERT(resource_id < DEMO_NUM_RESOURCES && resource_id >= 0);

    lthread_mutex_lock(&mutex);
    switch (resource_id)
    {
    case 0 :
        if (resource_1.available)
        {
            resource_1.available = 0;
            res = &resource_1;
        }
        break;

    case 1 :
        if (resource_2.free)
        {
            resource_2.free = 0;
            res = &resource_2;
        }
        break;
    }
    lthread_mutex_unlock(&mutex);
    return res;
}

static int put_resource (int manager_id, int resource_id, void * arg)
{
    ASSERT(DEMO_RESOURCE_MANAGER_ID == manager_id);
    ASSERT(resource_id < DEMO_NUM_RESOURCES && resource_id >= 0);

    lthread_mutex_lock(&mutex);
    switch (resource_id)
    {
    case 0 :
        ASSERT(resource_1.available == 0);
        resource_1.available = 1;
        break;

    case 1 :
        ASSERT(resource_2.free == 0);
        resource_2.free = 1;
        break;
    }
    lthread_mutex_unlock(&mutex);

    return 0;
}

void LIM_ResourceManagerInit (void)
{
    LIMDBG("init resource manager.\n");
    int resource_id[DEMO_NUM_RESOURCES + 1];
    int ii;

    // resourece id can be any value that is known to resourece manager.
    for (ii = 0; ii < DEMO_NUM_RESOURCES; ii++)
    {
        resource_id[ii] = ii;
    }
    resource_id[ii] = -1;

    resource_1.available = 1; //internal resource management flag
    resource_2.free = 1; //internal resource management flag

    //register resource with LIM resource management system.
    //this can be called multiple times to register multiple resources,
    //either with different manager ID or reuse the same manager ID, either
    //with different get/put resource callbacks or reuse the same.
    //the only requirement is that with the same manager ID, the resource ID
    //must be unique.
    limoi_resource_register(DEMO_RESOURCE_MANAGER_ID,
                            resource_id,
                            get_resource,
                            put_resource,
                            NULL);
}

void LIM_ResourceManagerDeinit (void)
{
    LIMDBG("deinit resource manager.\n");
    //unregister all resources from LIM resource management system.
    limoi_resource_unregister(DEMO_RESOURCE_MANAGER_ID);
}
