/*
  Copyright (C) 2011 SWOAG Technology <www.swoag.com>

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  LIM directory entry helper routines.

  REVISION:

    1) initial creation. ----------------------------------------- 2011-05-25 MG
*/

#define _GNU_SOURCE
#include "limlog.h"
#include "limdir.h"

#undef LIMDBG
#ifndef LIMDBG
#define LIMDBG(fmt, arg...)
#endif

static const char * get_next_file (DIR * dir)
{
    const char * fname = NULL;
    struct dirent * dirent;

    while (1)
    {
        dirent = readdir(dir);
        if (NULL == dirent) break;

        if (dirent->d_type != DT_REG && dirent->d_type != DT_LNK) continue;
        fname = dirent->d_name;
        break;
    }

    return fname;
}

static const char * get_next_dir (DIR * dir)
{
    const char * dname = NULL;
    struct dirent * dirent;

    while (1)
    {
        dirent = readdir(dir);
        if (NULL == dirent) break;

        if (dirent->d_type != DT_DIR) continue;
        if (!strcmp(dirent->d_name, ".")
            || !strcmp(dirent->d_name, "..")) continue;

        dname = dirent->d_name;
        break;
    }

    return dname;
}

static int dir_scan (
    const char * path,
    void (*next) (const char * ent, void * arg),
    void * arg)
{
    DIR * dir;
    const char * cname;
    char full_path_name[PATH_MAX];
    int len;

    dir = opendir(path);
    if (NULL == dir)
    {
        LIMDBG("unable to open directory: %s\n", path);
        return 0;
    }

    // append '/' if not present
    len = strlen(path);
    memcpy(full_path_name, path, len + 1);
    if (path[len - 1] != '/')
    {
        full_path_name[len] = '/';
        len++;
    }

    while (1)
    {
        cname = get_next_file(dir);
        if (NULL == cname) break;

        full_path_name[len] = 0;

        next(strcat(full_path_name, cname), arg);
    }

    closedir(dir);

    return 0;
}

int limdir_recursive_scan (
    const char * path,
    void (*next) (const char * ent, void * arg),
    void * arg)
{
    DIR * dir;
    const char * dname;
    char full_path_name[PATH_MAX];
    int len;

    ASSERT(path);

    dir = opendir(path);
    if (NULL == dir)
    {
        LIMDBG("unable to open directory: %s\n", path);
        return 0;
    }

    dir_scan(path, next, arg);

    // append '/' if not present
    len = strlen(path);
    memcpy(full_path_name, path, len + 1);
    if (path[len - 1] != '/')
    {
        full_path_name[len] = '/';
        len++;
    }

    while (1)
    {
        dname = get_next_dir(dir);
        if (NULL == dname) break;
        full_path_name[len] = 0;
        limdir_recursive_scan(strcat(full_path_name, dname), next, arg);
    }

    closedir(dir);
    return 0;
}
